@@ -38,6 +38,8 @@ Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
 #include "toplev.h"
 #include "tree-pass.h"
 
+/* LLVM LOCAL begin comment out most of this file */
+#ifndef ENABLE_LLVM
 /* Target register optimizations - these are performed after reload.  */
 
 typedef struct btr_def_group_s
@@ -1492,6 +1494,8 @@ branch_target_load_optimize (bool after_prologue_epilogue_gen)
 			PROP_DEATH_NOTES | PROP_REG_INFO);
     }
 }
+#endif
+/* LLVM LOCAL end */
 
 static bool
 gate_handle_branch_target_load_optimize (void)
@@ -1503,12 +1507,10 @@ gate_handle_branch_target_load_optimize (void)
 static unsigned int
 rest_of_handle_branch_target_load_optimize (void)
 {
-  static int warned = 0;
 /* LLVM LOCAL begin - reduce cc1 size.  */
-#ifdef ENABLE_LLVM
-  return 0;
-#endif
-/* LLVM LOCAL end.  */
+#ifndef ENABLE_LLVM
+  static int warned = 0;
+
   /* Leave this a warning for now so that it is possible to experiment
      with running this pass twice.  In 3.6, we should either make this
      an error, or use separate dump files.  */
@@ -1523,6 +1525,8 @@ rest_of_handle_branch_target_load_optimize (void)
     }
 
   branch_target_load_optimize (epilogue_completed);
+#endif
+/* LLVM LOCAL end.  */
   return 0;
 }
 
