@@ -1156,8 +1156,10 @@ enum {
      BLOCK_IS_GC =             (1 << 27),
      /* APPLE LOCAL radar 5822844 */
      BLOCK_IS_GLOBAL = 	       (1 << 28),
-     /* APPLE LOCAL radar 5847213 */
-     BLOCK_HAS_DESCRIPTOR =    (1 << 29)
+     /* APPLE LOCAL radar 7735196 */
+     BLOCK_USE_STRET =    (1 << 29),
+     /* APPLE LOCAL radar 8143947 */
+     BLOCK_HAS_SIGNATURE  =    (1 << 30)
 };
 
 struct block_sema_info {
@@ -1242,6 +1244,9 @@ extern tree build_block_helper_name (int);
 /* APPLE LOCAL radar 6353006  */
 extern tree c_build_generic_block_struct_type (void);
 
+/* APPLE LOCAL radar 7760213 */
+extern int HasByrefArray(tree);
+
 /* In c-omp.c  */
 extern tree c_finish_omp_master (tree);
 extern tree c_finish_omp_critical (tree, tree);
