@@ -368,13 +368,13 @@ define__GNUC__ (void)
   else
     builtin_define_with_value_n ("__GNUC_PATCHLEVEL__", "0", 1);
 
-  gcc_assert (!*v || *v == ' ' || *v == '-' || *v == '(');
+  gcc_assert (!*v || *v == ' ' || *v == '-');
 
   /* LLVM LOCAL no version number */
+#ifdef CONFIG_DARWIN_H
 #ifndef LLVM_VERSION_INFO
 
   /* APPLE LOCAL begin Apple version */
-#ifdef CONFIG_DARWIN_H
   {
     /* This chunk of code defines __APPLE_CC__ from the version
        string.  It expects to see a substring of the version string of
@@ -398,12 +398,10 @@ define__GNUC__ (void)
       abort ();
     builtin_define_with_value_n ("__APPLE_CC__", vt, q - vt);
   }
-#endif
   /* APPLE LOCAL end Apple version */
 
   /* LLVM LOCAL begin version number */
 #else
-#ifdef CONFIG_DARWIN_H
   /* This chunk of code defines __APPLE_CC__ from the version
      string.  It expects to see a substring of the version string of
      the form "build NNNN)", where each N is a digit, and the first
@@ -443,8 +441,8 @@ define__GNUC__ (void)
     else
       builtin_define_with_value_n ("__APPLE_CC__", vt, q - vt);
   }
-#endif /*CONFIG_DARWIN_H*/
 #endif /*LLVM_VERSION_INFO*/
+#endif /*CONFIG_DARWIN_H*/
   /* LLVM LOCAL end version number */
 }
 
@@ -474,7 +472,6 @@ c_cpp_builtins (cpp_reader *pfile)
     return;
 
   define__GNUC__ ();
-  
   /* LLVM LOCAL begin */
 #ifdef ENABLE_LLVM
   cpp_define (pfile, "__llvm__");
@@ -585,6 +582,11 @@ c_cpp_builtins (cpp_reader *pfile)
 
   /* Other target-independent built-ins determined by command-line
      options.  */
+  /* APPLE LOCAL begin blocks */
+  /* APPLE LOCAL radar 5868913 */
+  if (flag_blocks)
+    cpp_define (pfile, "__BLOCKS__=1"); 
+  /* APPLE LOCAL end blocks */
   if (optimize_size)
     cpp_define (pfile, "__OPTIMIZE_SIZE__");
   if (optimize)
@@ -596,7 +598,8 @@ c_cpp_builtins (cpp_reader *pfile)
     cpp_define (pfile, "__NO_INLINE__");
   if (flag_signaling_nans)
     cpp_define (pfile, "__SUPPORT_SNAN__");
-  if (flag_finite_math_only)
+  /* LLVM LOCAL */
+  if (!flag_honor_infinites & !flag_honor_nans)
     cpp_define (pfile, "__FINITE_MATH_ONLY__=1");
   else
     cpp_define (pfile, "__FINITE_MATH_ONLY__=0");
