@@ -372,7 +372,8 @@ cb_undef (cpp_reader * ARG_UNUSED (pfile), source_location loc,
    non-NULL.  */
 
 enum cpp_ttype
-c_lex_with_flags (tree *value, location_t *loc, unsigned char *cpp_flags)
+/* APPLE LOCAL CW asm blocks C++ comments 6338079 */
+c_lex_with_flags (tree *value, location_t *loc, unsigned char *cpp_flags, int defer)
 {
   static bool no_more_pch;
   const cpp_token *tok;
@@ -474,11 +475,21 @@ c_lex_with_flags (tree *value, location_t *loc, unsigned char *cpp_flags)
 
     case CPP_NUMBER:
       {
-	unsigned int flags = cpp_classify_number (parse_in, tok);
+	/* APPLE LOCAL CW asm blocks C++ comments 6338079 */
+	unsigned int flags = cpp_classify_number (parse_in, tok, defer);
 
 	switch (flags & CPP_N_CATEGORY)
 	  {
 	  case CPP_N_INVALID:
+	    /* APPLE LOCAL begin CW asm blocks C++ comments 6338079 */
+	    if (flags & CPP_N_DEFER)
+	      {
+		add_flags = ERROR_DEFERRED;
+		*value = error_mark_node;
+		break;
+	      }
+	    /* APPLE LOCAL end CW asm blocks C++ comments 6338079 */
+
 	    /* cpplib has issued an error.  */
 	    *value = error_mark_node;
 	    errorcount++;
@@ -607,7 +618,7 @@ c_lex_with_flags (tree *value, location_t *loc, unsigned char *cpp_flags)
 	/* Because we don't recognize inline asm commments during
 	   lexing, we have to pass this back to the parser to error
 	   out with or eat as a comment as appropriate.  */
-	if (flag_iasm_blocks_local)
+	if (defer && flag_iasm_blocks_local)
 	  {
 	    *value = build_int_cst_wide (char_type_node, c, 0);
 	    break;
@@ -694,7 +705,12 @@ c_lex_with_flags (tree *value, location_t *loc, unsigned char *cpp_flags)
     /* APPLE LOCAL begin 4137741 */
   if (!no_more_pch
       && type != CPP_BINCL
-      && type != CPP_EINCL)
+      && type != CPP_EINCL
+      /* APPLE LOCAL begin distcc pch 7216809 */
+      && type != CPP_PRAGMA
+      && type != CPP_STRING
+      && type != CPP_PRAGMA_EOL)
+      /* APPLE LOCAL end distcc pch 7216809 */
     /* APPLE LOCAL end 4137741 */
     {
       no_more_pch = true;
@@ -766,6 +782,7 @@ narrowest_signed_type (unsigned HOST_WIDE_INT low,
 }
 
 /* LLVM LOCAL begin hack for PR1521 */
+#ifdef ENABLE_LLVM
 #define PART_PRECISION (sizeof (cpp_num_part) * CHAR_BIT)
 /* Sign extend a number, with PRECISION significant bits and all
    others assumed clear, to fill out a cpp_num structure.  */
@@ -791,6 +808,7 @@ my_cpp_num_sign_extend (cpp_num num, size_t precision)
 
   return num;
 }
+#endif
 /* LLVM LOCAL end */
 
 /* Interpret TOKEN, an integer with FLAGS as classified by cpplib.  */
@@ -803,8 +821,13 @@ interpret_integer (const cpp_token *token, unsigned int flags)
   cpp_options *options = cpp_get_options (parse_in);
 
   integer = cpp_interpret_integer (parse_in, token, flags);
-  /* LLVM LOCAL hack for PR1521 */
+  /* LLVM LOCAL begin - hack for PR1521 */
+#ifdef ENABLE_LLVM
   integer = my_cpp_num_sign_extend (integer, options->precision);
+#else
+  integer = cpp_num_sign_extend (integer, options->precision);
+#endif
+  /* LLVM LOCAL end - hack for PR1521 */
 
   /* The type of a constant with a U suffix is straightforward.  */
   if (flags & CPP_N_UNSIGNED)
