@@ -168,6 +168,8 @@ c_common_missing_argument (const char *opt, size_t code)
     case OPT_isysroot:
     case OPT_isystem:
     case OPT_iquote:
+    /* APPLE LOCAL ARM iwithsysroot 4917039 */
+    case OPT_iwithsysroot:
       error ("missing path after %qs", opt);
       break;
 
@@ -236,6 +238,11 @@ c_common_init_options (unsigned int argc, const char **argv)
   /* APPLE LOCAL end -Wfour-char-constants  */
 
   flag_exceptions = c_dialect_cxx ();
+  /* LLVM local begin One Definition Rule */
+#ifdef ENABLE_LLVM
+  flag_odr = c_dialect_cxx ();
+#endif
+  /* LLVM local end */
   warn_pointer_arith = c_dialect_cxx ();
   warn_write_strings = c_dialect_cxx();
 
@@ -658,6 +665,11 @@ c_common_handle_option (size_t scode, const char *arg, int value)
 
     case OPT_fbuiltin:
       flag_no_builtin = !value;
+      /* LLVM LOCAL begin */
+#ifdef ENABLE_LLVM
+      flag_no_simplify_libcalls = !value;
+#endif
+      /* LLVM LOCAL end */
       break;
 
     case OPT_fbuiltin_:
@@ -683,6 +695,11 @@ c_common_handle_option (size_t scode, const char *arg, int value)
     case OPT_fhosted:
       flag_hosted = value;
       flag_no_builtin = !value;
+      /* LLVM LOCAL begin */
+#ifdef ENABLE_LLVM
+      flag_no_simplify_libcalls = !value;
+#endif
+      /* LLVM LOCAL end */
       /* warn_main will be 2 if set by -Wall, 1 if set by -Wmain */
       if (!value && warn_main == 2)
 	warn_main = 0;
@@ -773,6 +790,12 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       flag_implicit_templates = value;
       break;
 
+      /* APPLE LOCAL begin 5612787 mainline sse4 */
+    case OPT_flax_vector_conversions:
+      flag_lax_vector_conversions = value;
+      break;
+      /* APPLE LOCAL end 5612787 mainline sse4 */
+
     case OPT_fms_extensions:
       flag_ms_extensions = value;
       break;
@@ -912,6 +935,15 @@ c_common_handle_option (size_t scode, const char *arg, int value)
       sysroot = arg;
       break;
 
+    /* APPLE LOCAL begin ARM iwithsysroot 4917039 */
+    case OPT_iwithsysroot:
+      if (arg[0] != '/' || !sysroot)
+	add_path (xstrdup (arg), SYSTEM, 0, true);
+      else
+	add_path (concat (sysroot, arg, NULL), SYSTEM, 0, true);
+      break;
+    /* APPLE LOCAL end ARM iwithsysroot 4917039 */
+
     case OPT_isystem:
       add_path (xstrdup (arg), SYSTEM, 0, true);
       break;
@@ -1097,6 +1129,13 @@ c_common_post_options (const char **pfilename)
   if (flag_inline_functions)
     flag_inline_trees = 2;
 
+  /* APPLE LOCAL begin radar 5811887  - radar 6084601 */
+  /* In all flavors of c99, except for ObjC/ObjC++, blocks are off by default 
+     unless requested via -fblocks. */
+  if (flag_blocks == -1 && flag_iso && !c_dialect_objc())
+    flag_blocks = 0;
+  /* APPLE LOCAL end radar 5811887 - radar 6084601 */
+
   /* APPLE LOCAL begin mainline */
   /* By default we use C99 inline semantics in GNU99 or C99 mode.  C99
      inline semantics are not supported in GNU89 or C89 mode.  */
@@ -1153,8 +1192,11 @@ c_common_post_options (const char **pfilename)
 	       "-Wformat-zero-length ignored without -Wformat");
       warning (OPT_Wformat_nonliteral,
 	       "-Wformat-nonliteral ignored without -Wformat");
-      warning (OPT_Wformat_security,
-	       "-Wformat-security ignored without -Wformat");
+      /* LLVM LOCAL begin */
+      if (warn_format_security)
+        warning (OPT_Wformat_security,
+                 "-Wformat-security ignored without -Wformat");
+      /* LLVM LOCAL end */
     }
 
   /* C99 requires special handling of complex multiplication and division;
