@@ -36,10 +36,6 @@
 %{!fbuiltin-strcpy:-fno-builtin-strcpy} \
 %<fbuiltin-strcat \
 %<fbuiltin-strcpy \
-"/* LLVM LOCAL ignore -g in LTO mode */"\
-"/* On Darwin, debug info is stored in separate .dSYM files. */"\
-"/* This requires special support in LTO mode. */" \
-%{O4|flto: %<g* } \
 %<pg"
 
 #undef LIB_SPEC
@@ -251,7 +247,11 @@
 #define DARWIN_LD_MINVERSION_SPEC "-iphoneos_version_min %(darwin_minversion)"
 
 /* Use iPhone OS version numbers by default.  */
+#ifdef DEFAULT_TARGET_OS
+#define DARWIN_DEFAULT_VERSION_TYPE  DEFAULT_TARGET_OS
+#else
 #define DARWIN_DEFAULT_VERSION_TYPE  DARWIN_VERSION_IPHONEOS
+#endif
 
 #define DARWIN_IPHONEOS_LIBGCC_SPEC "-lgcc_s.1 -lgcc"
 
@@ -328,8 +328,10 @@ do {									\
 #define SUBTARGET_ASM_DECLARE_FUNCTION_NAME ARM_DECLARE_FUNCTION_NAME
 
 /* APPLE LOCAL begin 6093388 -mfpu=neon default for v7a */
-/* We default to VFP for v6, NEON for v7 */
-#define FPUTYPE_DEFAULT (arm_arch7a ? FPUTYPE_NEON : FPUTYPE_VFP)
+/* We default to VFP for v6, NEON for v7. */
+#define FPUTYPE_DEFAULT (arm_arch7a ? "neon" : \
+                         ((arm_tune == cortexm3 || arm_tune == cortexm0) ? \
+                          "fpe2" : "vfp"))
 
 #undef TARGET_DEFAULT_FLOAT_ABI
 #define TARGET_DEFAULT_FLOAT_ABI ((arm_arch6 || arm_arch7a) ? ARM_FLOAT_ABI_SOFTFP : ARM_FLOAT_ABI_SOFT)
