@@ -94,6 +94,10 @@ enum darwin_builtins
 int darwin_reverse_bitfields = 0;
 /* APPLE LOCAL end pragma reverse_bitfields */
 
+/* APPLE LOCAL begin axe stubs 5571540 */
+int darwin_stubs = false;
+/* APPLE LOCAL end axe stubs 5571540 */
+
 /* Section names.  */
 section * darwin_sections[NUM_DARWIN_SECTIONS];
 
@@ -139,12 +143,15 @@ output_objc_section_asm_op (const void *directive)
 	  objc_class_ext_section,
 	  objc_prop_list_section
 	  /* APPLE LOCAL end objc2 */
-	  /* APPLE LOCAL curly braces confuse me */
 	};
       /* APPLE LOCAL begin ObjC abi v2 - radar 4792158 */
       static const enum darwin_section_enum tomarkv2[] =
 	{
 	  objc_v2_message_refs_section,
+	  /* APPLE LOCAL begin radar 6255595 */
+	  objc_v2_classdefs_section,
+	  objc_v2_metadata_section,
+	  /* APPLE LOCAL end radar 6255595 */
 	  objc_v2_classrefs_section,
 	  objc_v2_classlist_section,
 	  objc_v2_categorylist_section,
@@ -156,7 +163,7 @@ output_objc_section_asm_op (const void *directive)
 	  objc_v2_super_classrefs_section,
 	  objc_v2_image_info_section,
 	  objc_v2_constant_string_object_section
-	};
+	} ;
       /* APPLE LOCAL end ObjC abi v2 - radar 4792158 */
       size_t i;
 
@@ -226,6 +233,15 @@ machopic_symbol_defined_p (rtx sym_ref)
 	  if (DECL_COMMON (decl))
 	    return false;
 	}
+      /* APPLE LOCAL begin 6077274 */
+      /* Weak functions should always be indirected.  */
+      else if (SYMBOL_REF_FLAGS (sym_ref) & SYMBOL_FLAG_FUNCTION)
+	{
+	  tree decl = SYMBOL_REF_DECL (sym_ref);
+	  if (decl && DECL_WEAK (decl))
+	    return false;
+	}
+      /* APPLE LOCAL end 6077274 */
       return true;
     }
   return false;
@@ -264,7 +280,8 @@ machopic_classify_symbol (rtx sym_ref)
    permit the runtime to rebind new instances of the translation unit
    to the original instance of the data.  */
 
-static int
+/* APPLE LOCAL fix-and-continue 6227434 */
+int
 indirect_data (rtx sym_ref)
 {
   int lprefix;
@@ -282,8 +299,14 @@ indirect_data (rtx sym_ref)
   name = XSTR (sym_ref, 0);
 
   lprefix = (((name[0] == '*' || name[0] == '&')
-              && (name[1] == 'L' || (name[1] == '"' && name[2] == 'L')))
-             || (strncmp (name, "_OBJC_", 6) == 0));
+	      /* APPLE LOCAL begin fix-and-continue 6227434 */
+              && (name[1] == 'L'
+		  || (name[1] == '"' && name[2] == 'L')
+		  /* Don't indirect writable strings.  */
+		  || (name[1] == 'l' && name[2] == 'C')))
+             || (strncmp (name, "_OBJC_", 6) == 0)
+	     || objc_anonymous_local_objc_name (name));
+	      /* APPLE LOCAL end fix-and-continue 6227434 */
 
   return ! lprefix;
 }
@@ -789,6 +812,11 @@ machopic_force_indirect_call_target (rtx target)
 rtx
 machopic_indirect_call_target (rtx target)
 {
+  /* APPLE LOCAL begin axe stubs 5571540 */
+  if (! darwin_stubs)
+    return target;
+  /* APPLE LOCAL end axe stubs 5571540 */
+
   if (GET_CODE (target) != MEM)
     return target;
 
@@ -1308,6 +1336,30 @@ machopic_reloc_rw_mask (void)
   return MACHOPIC_INDIRECT ? 3 : 0;
 }
 
+/* APPLE LOCAL begin radar 5575115, 6255595 */
+/* This routine returns TRUE if EXP is a variable representing
+   on objective C meta data. */
+static inline bool
+objc_internal_variable_name (tree exp)
+{
+  if (TREE_CODE (exp) == VAR_DECL)
+    {
+      tree decl_name = DECL_NAME (exp);
+      if (decl_name && TREE_CODE (decl_name) == IDENTIFIER_NODE
+	  && IDENTIFIER_POINTER (decl_name))
+	{
+	  const char* name = IDENTIFIER_POINTER (decl_name);
+	  return 
+	   (!strncmp (name, "_OBJC_", 6)
+	    || !strncmp (name, "OBJC_", 5)
+	    || !strncmp (name, "l_OBJC_", 7)
+            || !strncmp (name, "l_objc_", 7));
+	}
+    }
+  return false;
+}
+/* APPLE LOCAL end radar 5575115, 6255595 */
+
 section *
 machopic_select_section (tree exp, int reloc,
 			 unsigned HOST_WIDE_INT align ATTRIBUTE_UNUSED)
@@ -1336,12 +1388,14 @@ machopic_select_section (tree exp, int reloc,
 
   /* APPLE LOCAL begin fwritable strings  */
   if (TREE_CODE (exp) == STRING_CST
-      && ((size_t) TREE_STRING_LENGTH (exp)
+      /* APPLE LOCAL begin 5612787 mainline sse4 */
+      /* deletion */
       /* Copied from varasm.c:output_constant_def_contents().  5346453 */
       && (MAX ((HOST_WIDE_INT)TREE_STRING_LENGTH (exp),
 	       int_size_in_bytes (TREE_TYPE (exp)))
 	  /* APPLE LOCAL ARM signedness mismatch */
-	  == (HOST_WIDE_INT) strlen (TREE_STRING_POINTER (exp)) + 1))
+	  == (HOST_WIDE_INT) strlen (TREE_STRING_POINTER (exp)) + 1)
+      /* APPLE LOCAL end 5612787 mainline sse4 */
       && ! flag_writable_strings)
     return darwin_sections[cstring_section];
   /* APPLE LOCAL end fwritable strings, 5346453 */
@@ -1404,11 +1458,9 @@ machopic_select_section (tree exp, int reloc,
       else
 	return base_section;
     }
-  else if (TREE_CODE (exp) == VAR_DECL &&
-	   DECL_NAME (exp) &&
-	   TREE_CODE (DECL_NAME (exp)) == IDENTIFIER_NODE &&
-	   IDENTIFIER_POINTER (DECL_NAME (exp)) &&
-	   !strncmp (IDENTIFIER_POINTER (DECL_NAME (exp)), "_OBJC_", 6))
+  /* APPLE LOCAL begin radar 5575115, 6255595 */
+  else if (objc_internal_variable_name (exp))
+  /* APPLE LOCAL end radar 5575115, 6255595 */
     {
       const char *name = IDENTIFIER_POINTER (DECL_NAME (exp));
       /* APPLE LOCAL begin radar 4792158 */
@@ -1483,11 +1535,13 @@ machopic_select_section (tree exp, int reloc,
             return darwin_sections[objc_v2_classrefs_section];
           else if (!strncmp (name, "_OBJC_CLASSLIST_SUP_REFS_", 25))
             return darwin_sections[objc_v2_super_classrefs_section];
-          else if (!strncmp (name, "_OBJC_MESSAGE_REF", 17))
+          /* APPLE LOCAL radar 5575115 - radar 6252174 */
+          else if (!strncmp (name, "l_objc_msgSend", 14))
             return darwin_sections[objc_v2_message_refs_section];
           else if (!strncmp (name, "_OBJC_LABEL_CLASS_", 18))
             return darwin_sections[objc_v2_classlist_section];
-          else if (!strncmp (name, "_OBJC_LABEL_PROTOCOL_", 21))
+          /* APPLE LOCAL radar 6351990 */
+          else if (!strncmp (name, "l_OBJC_LABEL_PROTOCOL_", 22))
             return darwin_sections[objc_v2_protocollist_section];
           else if (!strncmp (name, "_OBJC_LABEL_CATEGORY_", 21))
             return darwin_sections[objc_v2_categorylist_section];
@@ -1495,14 +1549,21 @@ machopic_select_section (tree exp, int reloc,
             return darwin_sections[objc_v2_nonlazy_class_section];
           else if (!strncmp (name, "_OBJC_LABEL_NONLAZY_CATEGORY_", 29))
             return darwin_sections[objc_v2_nonlazy_category_section];
-          else if (!strncmp (name, "_OBJC_PROTOCOL_REFERENCE_", 25))
+          /* APPLE LOCAL radar 6351990 */
+          else if (!strncmp (name, "l_OBJC_PROTOCOL_REFERENCE_", 26))
             return darwin_sections[objc_v2_protocolrefs_section];
           else if (!strncmp (name, "_OBJC_SELECTOR_REFERENCES", 25))
             return darwin_sections[objc_v2_selector_refs_section];
           else if (!strncmp (name, "_OBJC_IMAGE_INFO", 16))
             return darwin_sections[objc_v2_image_info_section];
+	    /* APPLE LOCAL begin radar 6255595 */
+	  else if (!strncmp (name, "OBJC_CLASS_$_", 13)
+		   || !strncmp (name, "OBJC_METACLASS_$_", 17))
+	    return darwin_sections[objc_v2_classdefs_section];
           else
-            return base_section;
+            return  (base_section == data_section) ? 
+		      darwin_sections[objc_v2_metadata_section] : base_section;
+    	  /* APPLE LOCAL end radar 6255595 */
 	}
       /* APPLE LOCAL end radar 4792158 */
     }
@@ -1515,10 +1576,181 @@ machopic_select_section (tree exp, int reloc,
 }
 
 /* LLVM LOCAL begin */
-#ifdef ENABLE_LLVM
-const char *darwin_objc_llvm_implicit_target_global_var_section(tree decl) {
-  const char *name;
+extern char * mempcpy (char *dst, const char *src, size_t len);
+char *darwin_build_sysroot_path(const char *sysroot, const char *path) {
+  char *str = NULL;
+  char *str1 = NULL;
+  char *darwin = NULL;
+#ifndef ENABLE_LLVM
+  return concat (sysroot, path, NULL);
+#endif
 
+  /* FIXME : When time is appropriate, handle other sdks.  */
+  if (sysroot && strstr(sysroot, "MacOSX10.5.sdk") == NULL)
+    return concat (sysroot, path, NULL);
+
+  /* libstdc++ headers are fixed magically through sym link jungle.  */
+  if (strstr(path, "c++") != NULL)
+    return concat (sysroot, path, NULL);
+
+  darwin = strstr(path, "apple-darwin");
+  if (!darwin)
+    return concat (sysroot, path, NULL);
+
+  /* Released 10.5 SDK uses header paths that include OS version
+     number, for example 9 in 
+     .../MacOSX10.5.sdk/.../lib/gcc/i686-apple-darwin9/4.2.1/include
+     However the 9 is constructed based on the host OS version on
+     which the compiler is built. This means, the compiler will
+     not be able to use 10.5 SDK unless it is built on 10.5 system.
+     Fix header path here to make it work.
+     
+     Path includes "apple-darwinXYZ/" substring. Replace
+     this substring with "apple-darwin9/". */
+  str = XNEWVEC(char, strlen(sysroot) + strlen(path) + 2);
+  str1 = mempcpy(str, sysroot, strlen(sysroot));
+  str1 = mempcpy(str1, path, darwin - path);
+  str1 = mempcpy(str1, "apple-darwin9", strlen("apple-darwin9"));
+  darwin = strchr(darwin, '/');
+  str1 = mempcpy(str1, darwin, strlen(darwin));
+  return str;
+}
+
+#ifdef ENABLE_LLVM
+static const char *skip_objc_prefix(const char *name)
+{
+  if (!strncmp (name, "_OBJC_", 6))
+    return name + 6;
+  else if (!strncmp (name, "OBJC_", 5))
+    return name + 5;
+
+  return name + 7;
+}
+
+static const char *
+darwin_objc_llvm_special_name_section_help(tree decl) {
+  /* Get a pointer to the name, past the L_OBJC_ prefix. */
+  const char *name = IDENTIFIER_POINTER (DECL_NAME (decl));
+  const char *base_section = 0;
+  const char *section = 0;
+  bool weak_p = (DECL_P (decl) && DECL_WEAK (decl)
+		 && (lookup_attribute ("weak", DECL_ATTRIBUTES (decl))
+		     || ! lookup_attribute ("weak_import",
+					    DECL_ATTRIBUTES (decl))));
+
+  if (TREE_READONLY (decl) || TREE_CONSTANT (decl))
+    base_section = weak_p ? "__DATA,__const_coal,coalesced" : "__DATA,__const";
+  else
+    base_section = weak_p ? "__DATA,__datacoal_nt,coalesced" : "__DATA,__data";
+
+  name = skip_objc_prefix(name);
+  section = darwin_objc_llvm_special_name_section(name);
+
+  if (!section && flag_objc_abi == 2)
+    section = strcmp(base_section, "__DATA,__data") == 0 ?
+      "__DATA, __objc_const" : base_section;
+
+  return section ? section : base_section;
+}
+
+const char *darwin_objc_llvm_special_name_section(const char *name) {
+  if (flag_objc_abi == 1) {
+    if (!strncmp (name, "CLASS_METHODS_", 14))
+      return "__OBJC,__cls_meth,regular,no_dead_strip";
+    else if (!strncmp (name, "INSTANCE_METHODS_", 17))
+      return "__OBJC,__inst_meth,regular,no_dead_strip";
+    else if (!strncmp (name, "CATEGORY_CLASS_METHODS_", 23))
+      return "__OBJC,__cat_cls_meth,regular,no_dead_strip";
+    else if (!strncmp (name, "CATEGORY_INSTANCE_METHODS_", 26))
+      return "__OBJC,__cat_inst_meth,regular,no_dead_strip";
+    else if (!strncmp (name, "CLASS_VARIABLES_", 16))
+      return "__OBJC,__class_vars,regular,no_dead_strip";
+    else if (!strncmp (name, "INSTANCE_VARIABLES_", 19))
+      return "__OBJC,__instance_vars,regular,no_dead_strip";
+    else if (!strncmp (name, "CLASS_PROTOCOLS_", 16))
+      return "__OBJC,__cat_cls_meth,regular,no_dead_strip";
+    else if (!strncmp (name, "CLASS_NAME_", 11))
+      return "__TEXT,__cstring,cstring_literals";
+    else if (!strncmp (name, "METH_VAR_NAME_", 14))
+      return "__TEXT,__cstring,cstring_literals";
+    else if (!strncmp (name, "METH_VAR_TYPE_", 14))
+      return "__TEXT,__cstring,cstring_literals";
+    else if (!strncmp (name, "PROP_NAME_ATTR_", 15))
+      return "__TEXT,__cstring,cstring_literals";
+    else if (!strncmp (name, "CLASS_REFERENCES", 16))
+      return "__OBJC,__cls_refs,literal_pointers,no_dead_strip";
+    else if (!strncmp (name, "CLASS_", 6))
+      return "__OBJC,__class,regular,no_dead_strip";
+    else if (!strncmp (name, "METACLASS_", 10))
+      return "__OBJC,__meta_class,regular,no_dead_strip";
+    else if (!strncmp (name, "CATEGORY_", 9))
+      return "__OBJC,__category,regular,no_dead_strip";
+    else if (!strncmp (name, "SELECTOR_REFERENCES", 19))
+      return "__OBJC,__message_refs,literal_pointers,no_dead_strip";
+    else if (!strncmp (name, "SELECTOR_FIXUP", 14))
+      return "__OBJC,__sel_fixup,regular";/*,no_dead_strip";*/
+    else if (!strncmp (name, "SYMBOLS", 7))
+      return "__OBJC,__symbols,regular,no_dead_strip";
+    else if (!strncmp (name, "MODULES", 7))
+      return "__OBJC,__module_info,regular,no_dead_strip";
+    else if (!strncmp (name, "IMAGE_INFO", 10))
+      return "__OBJC, __image_info,regular" /*,no_dead_strip";*/;
+    else if (!strncmp (name, "PROTOCOL_INSTANCE_METHODS_", 26))
+      return "__OBJC,__cat_inst_meth,regular,no_dead_strip";
+    else if (!strncmp (name, "PROTOCOL_CLASS_METHODS_", 23))
+      return "__OBJC,__cat_cls_meth,regular,no_dead_strip";
+    else if (!strncmp (name, "PROTOCOL_REFS_", 14))
+      return "__OBJC,__cat_cls_meth,regular,no_dead_strip";
+    else if (!strncmp (name, "PROTOCOL_", 9))
+      return "__OBJC,__protocol,regular,no_dead_strip";
+    else if (!strncmp (name, "CLASSEXT_", 9))
+      return "__OBJC,__class_ext,regular,no_dead_strip";
+    else if (!strncmp (name, "$_PROP_LIST", 11)
+             || !strncmp (name, "$_PROP_PROTO", 12))
+      return "__OBJC,__property,regular,no_dead_strip";
+    else if (!strncmp (name, "PROTOCOLEXT", 11))
+      return "__OBJC,__protocol_ext,regular,no_dead_strip";
+    else if (!strncmp (name, "PROP_NAME_ATTR_", 15))
+      return "__TEXT,__cstring,cstring_literals";
+  } else if (flag_objc_abi == 2) {
+    if (!strncmp (name, "PROP_NAME_ATTR_", 15))
+      return "__TEXT,__cstring,cstring_literals";
+    else if (!strncmp (name, "CLASSLIST_REFERENCES_", 21))
+      return "__DATA, __objc_classrefs, regular, no_dead_strip";
+    else if (!strncmp (name, "CLASSLIST_SUP_REFS_", 19))
+      return "__DATA, __objc_superrefs, regular, no_dead_strip"; 
+    else if (!strncmp (name, "msgSend", 7))
+      return "__DATA, __objc_msgrefs, coalesced";
+    else if (!strncmp (name, "LABEL_CLASS_", 12))
+      return "__DATA, __objc_classlist, regular, no_dead_strip"; 
+    else if (!strncmp (name, "LABEL_PROTOCOL_", 15))
+      return "__DATA, __objc_protolist, coalesced, no_dead_strip"; 
+    else if (!strncmp (name, "LABEL_CATEGORY_", 15))
+      return "__DATA, __objc_catlist, regular, no_dead_strip"; 
+    else if (!strncmp (name, "LABEL_NONLAZY_CLASS_", 20))
+      return "__DATA, __objc_nlclslist, regular, no_dead_strip";
+    else if (!strncmp (name, "LABEL_NONLAZY_CATEGORY_", 23))
+      return "__DATA, __objc_nlcatlist, regular, no_dead_strip";
+    else if (!strncmp (name, "PROTOCOL_REFERENCE_", 19))
+      return "__DATA, __objc_protorefs, coalesced, no_dead_strip";
+    else if (!strncmp (name, "SELECTOR_REFERENCES", 19))
+      return "__DATA, __objc_selrefs, literal_pointers, no_dead_strip";
+    else if (!strncmp (name, "IMAGE_INFO", 10))
+      return "__DATA, __objc_imageinfo, regular, no_dead_strip";
+    else if (!strncmp (name, "CLASS_$_", 8)
+             || !strncmp (name, "METACLASS_$_", 12))
+      return "__DATA, __objc_data";
+    else if (!strncmp (name, "METH_VAR_NAME_", 14))
+      return "__TEXT, __objc_methname, cstring_literals";
+    else if (!strncmp (name, "METH_VAR_TYPE_", 14))
+      return "__TEXT, __objc_methtype, cstring_literals";
+    else if (!strncmp (name, "CLASS_NAME_", 11))
+      return "__TEXT, __objc_classname, cstring_literals";
+  }
+  return 0;
+}
+
+const char *darwin_objc_llvm_implicit_target_global_var_section(tree decl) {
   if (TREE_CODE(decl) == CONST_DECL) {
     extern int flag_next_runtime;
     tree typename = TYPE_NAME(TREE_TYPE(decl));
@@ -1527,92 +1759,21 @@ const char *darwin_objc_llvm_implicit_target_global_var_section(tree decl) {
     
     if (!strcmp(IDENTIFIER_POINTER(typename), "__builtin_ObjCString")) {
       if (flag_next_runtime)
-        return "__OBJC,__cstring_object,regular,no_dead_strip";
+        return "__OBJC, __cstring_object,regular,no_dead_strip";
       else
-        return "__OBJC,__string_object,no_dead_strip";
+        return "__OBJC, __string_object,no_dead_strip";
     } else if (!strcmp(IDENTIFIER_POINTER(typename), "__builtin_CFString")) {
-      return "__DATA,__cfstring";
+      return
+#ifdef LLVM_CONST_DATA_SECTION
+	(flag_writable_strings) ? LLVM_CONST_DATA_SECTION :
+#endif
+	"__DATA, __cfstring";
     } else {
       return 0;
     }
   }
   
-  /* Get a pointer to the name, past the L_OBJC_ prefix. */
-  name = IDENTIFIER_POINTER (DECL_NAME (decl))+7;
-  
-  if (!strncmp (name, "CLASS_METHODS_", 14))
-    return "__OBJC,__cls_meth,regular,no_dead_strip";
-  else if (!strncmp (name, "INSTANCE_METHODS_", 17))
-    return "__OBJC,__inst_meth,regular,no_dead_strip";
-  else if (!strncmp (name, "CATEGORY_CLASS_METHODS_", 23))
-    return "__OBJC,__cat_cls_meth,regular,no_dead_strip";
-  else if (!strncmp (name, "CATEGORY_INSTANCE_METHODS_", 26))
-    return "__OBJC,__cat_inst_meth,regular,no_dead_strip";
-  else if (!strncmp (name, "CLASS_VARIABLES_", 16))
-    return "__OBJC,__class_vars,regular,no_dead_strip";
-  else if (!strncmp (name, "INSTANCE_VARIABLES_", 19))
-    return "__OBJC,__instance_vars,regular,no_dead_strip";
-  else if (!strncmp (name, "CLASS_PROTOCOLS_", 16))
-    return "__OBJC,__cls_meth,regular,no_dead_strip";
-  else if (!strncmp (name, "CLASS_NAME_", 11))
-    return "__TEXT,__cstring,cstring_literals";
-  else if (!strncmp (name, "METH_VAR_NAME_", 14))
-    return "__TEXT,__cstring,cstring_literals";
-  else if (!strncmp (name, "METH_VAR_TYPE_", 14))
-    return "__TEXT,__cstring,cstring_literals";
-  else if (!strncmp (name, "CLASS_REFERENCES", 16))
-    return "__OBJC,__cls_refs,literal_pointers,no_dead_strip";
-  else if (!strncmp (name, "CLASS_", 6))
-    return "__OBJC,__class,regular,no_dead_strip";
-  else if (!strncmp (name, "METACLASS_", 10))
-    return "__OBJC,__meta_class,regular,no_dead_strip";
-  else if (!strncmp (name, "CATEGORY_", 9))
-    return "__OBJC,__category,regular,no_dead_strip";
-  else if (!strncmp (name, "SELECTOR_REFERENCES", 19))
-    return (flag_objc_abi == 1 ? 
-            "__OBJC,__message_refs,literal_pointers,no_dead_strip" :
-            "__DATA, __objc_selrefs, regular, no_dead_strip");
-  else if (!strncmp (name, "SELECTOR_FIXUP", 14))
-    return "__OBJC,__sel_fixup,regular";/*,no_dead_strip";*/
-  else if (!strncmp (name, "SYMBOLS", 7))
-    return "__OBJC,__symbols,regular,no_dead_strip";
-  else if (!strncmp (name, "MODULES", 7))
-    return "__OBJC,__module_info,regular,no_dead_strip";
-  else if (!strncmp (name, "IMAGE_INFO", 10))
-    return (flag_objc_abi == 1 ? 
-            "__OBJC,__image_info,regular" /*,no_dead_strip";*/ :
-            "__DATA, __objc_imageinfo, regular, no_dead_strip");
-  else if (!strncmp (name, "PROTOCOL_INSTANCE_METHODS_", 26))
-    return "__OBJC,__cat_inst_meth,regular,no_dead_strip";
-  else if (!strncmp (name, "PROTOCOL_CLASS_METHODS_", 23))
-    return "__OBJC,__cat_cls_meth,regular,no_dead_strip";
-  else if (!strncmp (name, "PROTOCOL_REFS_", 14))
-    return "__OBJC,__cat_cls_meth,regular,no_dead_strip";
-  else if (!strncmp (name, "PROTOCOL_", 9))
-    return "__OBJC,__protocol,regular,no_dead_strip";
-  else if (flag_objc_abi == 2) {
-    if (!strncmp (name, "CLASSLIST_REFERENCES_", 21))
-    return "__DATA, __objc_classrefs, regular, no_dead_strip";
-    else if (!strncmp (name, "CLASSLIST_SUP_REFS_", 19))
-      return "__DATA, __objc_superrefs, regular, no_dead_strip"; 
-    else if (!strncmp (name, "MESSAGE_REF", 11))
-      return "__DATA, __objc_msgrefs, regular, no_dead_strip"; 
-    else if (!strncmp (name, "LABEL_CLASS_", 12))
-      return "__DATA, __objc_classlist, regular, no_dead_strip"; 
-    else if (!strncmp (name, "LABEL_PROTOCOL_", 15))
-      return "__DATA, __objc_protolist, regular, no_dead_strip"; 
-    else if (!strncmp (name, "LABEL_CATEGORY_", 15))
-      return "__DATA, __objc_catlist, regular, no_dead_strip"; 
-    else if (!strncmp (name, "LABEL_NONLAZY_CLASS_", 20))
-      return "__DATA, __objc_nlclslist, regular, no_dead_strip";
-    else if (!strncmp (name, "LABEL_NONLAZY_CAGEGORY_", 23))
-      return "__DATA, __objc_nlcatlist, regular, no_dead_strip";
-    else if (!strncmp (name, "PROTOCOL_REFERENCE_", 19))
-      return "__DATA, __objc_protorefs, regular, no_dead_strip";
-    else 
-      return 0;
-  } else
-    return 0;
+  return darwin_objc_llvm_special_name_section_help(decl);
 }
 #endif
 /* LLVM LOCAL end */
@@ -1745,6 +1906,40 @@ darwin_handle_objc_gc_attribute (tree *node,
 }
 /* APPLE LOCAL end ObjC GC */
 
+/* APPLE LOCAL begin radar 5595352 */
+tree
+darwin_handle_nsobject_attribute (tree *node,
+                                  tree name,
+                                  tree args ATTRIBUTE_UNUSED,
+                                  int flags ATTRIBUTE_UNUSED,
+                                  bool *no_add_attrs)
+{
+  tree orig = *node, type;
+  if (!POINTER_TYPE_P (orig) || TREE_CODE (TREE_TYPE (orig)) != RECORD_TYPE)
+    {
+      error ("__attribute ((NSObject)) is for pointer types only");
+      return NULL_TREE;
+    }
+  type = build_type_attribute_variant (orig,
+				       tree_cons (name, NULL_TREE,
+				       TYPE_ATTRIBUTES (orig)));
+  /* The main variant must be preserved no matter what. What ever
+     main variant comes out of the call to build_type_attribute_variant
+     is bogus here. */
+  if (TYPE_MAIN_VARIANT (orig) != TYPE_MAIN_VARIANT (type))
+    {
+      TYPE_MAIN_VARIANT (type) = TYPE_MAIN_VARIANT (orig);
+      TYPE_NEXT_VARIANT (type) = TYPE_NEXT_VARIANT (orig);
+      TYPE_NEXT_VARIANT (orig) = type;
+    }
+
+  *node = type;
+  /* No need to hang on to the attribute any longer.  */
+  *no_add_attrs = true;
+  return NULL_TREE;
+}
+/* APPLE LOCAL end radar 5595352 */
+
 /* APPLE LOCAL begin darwin_set_section_for_var_p  20020226 --turly  */
 
 /* This is specifically for any initialised static class constants
@@ -1886,7 +2081,17 @@ darwin_handle_weak_import_attribute (tree *node, tree name,
   if (objc_method_decl (TREE_CODE (*node)))
     return NULL_TREE;
   /* APPLE LOCAL end radar 4733555 */
-  if (TREE_CODE (*node) != FUNCTION_DECL && TREE_CODE (*node) != VAR_DECL)
+  /* APPLE LOCAL begin weak_import on property 6676828 */
+  if (in_objc_property_decl_context ())
+    {
+      *no_add_attrs = true;
+      return NULL_TREE;
+    }
+  /* APPLE LOCAL end weak_import on property 6676828 */
+  /* LLVM LOCAL 6652529 begin */
+  if ((TREE_CODE (*node) != FUNCTION_DECL && TREE_CODE (*node) != VAR_DECL)
+      || !DECL_EXTERNAL (*node))
+  /* LLVM LOCAL 6652529 end */
     {
       warning (OPT_Wattributes, "%qs attribute ignored",
 	       IDENTIFIER_POINTER (name));
@@ -1985,16 +2190,21 @@ darwin_non_lazy_pcrel (FILE *file, rtx addr)
    VISIBILITY_INTERNAL or VISIBILITY_PROTECTED. */
 
 void
-darwin_assemble_visibility (tree decl, int vis)
+/* LLVM LOCAL begin */
+darwin_assemble_visibility (tree decl ATTRIBUTE_UNUSED, int vis)
 {
   if (vis == VISIBILITY_DEFAULT)
     ;
   else if (vis == VISIBILITY_HIDDEN)
     {
+/* LLVM LOCAL */
+#ifndef ENABLE_LLVM
       fputs ("\t.private_extern ", asm_out_file);
       assemble_name (asm_out_file,
 		     (IDENTIFIER_POINTER (DECL_ASSEMBLER_NAME (decl))));
       fputs ("\n", asm_out_file);
+/* LLVM LOCAL */
+#endif
     }
   else
     warning (OPT_Wattributes, "internal and protected visibility attributes "
@@ -2048,6 +2258,8 @@ darwin_file_start (void)
 	  /* APPLE LOCAL begin pubtypes, approved for 4.3 4535968  */
 	  DEBUG_PUBTYPES_SECTION,
 	  /* APPLE LOCAL end pubtypes, approved for 4.3 4535968  */
+	  /* APPLE LOCAL radar 6275985 debug inlined section  */
+	  DEBUG_INLINED_SECTION,
 	  DEBUG_STR_SECTION,
 	  DEBUG_RANGES_SECTION
 	};
@@ -2376,7 +2588,7 @@ darwin_build_constant_cfstring (tree str)
         {
           size_t numUniChars;
           const unsigned char *inbuf = (unsigned char *)TREE_STRING_POINTER (str);
-          utf16_str = objc_create_init_utf16_var (inbuf, length, &numUniChars);
+          utf16_str = create_init_utf16_var (inbuf, length, &numUniChars);
           if (!utf16_str)
             {
               warning (0, "input conversion stopped due to an input byte "
@@ -2424,6 +2636,7 @@ darwin_build_constant_cfstring (tree str)
       var = build_decl (CONST_DECL, NULL, TREE_TYPE (constructor));
       DECL_INITIAL (var) = constructor;
       TREE_STATIC (var) = 1;
+      DECL_IGNORED_P (var) = 1;
       pushdecl_top_level (var);
       desc->constructor = var;
     }
@@ -2523,6 +2736,8 @@ darwin_override_options (void)
 
       /* No EH in kexts.  */
       flag_exceptions = 0;
+      /* APPLE LOCAL 5628030 */
+      flag_asynchronous_unwind_tables = 0;
       /* No -fnon-call-exceptions data in kexts.  */
       flag_non_call_exceptions = 0;
       /* APPLE LOCAL begin kext v2 */
@@ -2531,6 +2746,38 @@ darwin_override_options (void)
 	flag_apple_kext = 2;
       /* APPLE LOCAL end kext v2 */
     }
+  /* APPLE LOCAL begin axe stubs 5571540 */
+  /* APPLE LOCAL begin ARM 5683689 */
+
+  /* Go ahead and generate stubs for old systems, just in case.  */
+  if (darwin_macosx_version_min
+      && strverscmp (darwin_macosx_version_min, "10.5") < 0)
+    darwin_stubs = true;
+  /* APPLE LOCAL end ARM 5683689 */
+  /* APPLE LOCAL end axe stubs 5571540 */
+  /* APPLE LOCAL begin stack-protector default 5095227 */
+  /* Default flag_stack_protect to 1 if on 10.5 or later for user code,
+     or 10.6 or later for code identified as part of the kernel.  */
+  /* LLVM LOCAL begin - Don't enable stack protectors by default for Leopard. */
+#ifndef ENABLE_LLVM
+  if (flag_stack_protect == -1
+      && darwin_macosx_version_min
+      && ((! flag_mkernel && ! flag_apple_kext
+	   && strverscmp (darwin_macosx_version_min, "10.5") >= 0)
+	  || strverscmp (darwin_macosx_version_min, "10.6") >= 0))
+    flag_stack_protect = 1;
+#else
+  if (flag_stack_protect == -1 && darwin_macosx_version_min)
+    {
+      if (strverscmp (darwin_macosx_version_min, "10.6") >= 0)
+        flag_stack_protect = 1;
+      else if (strverscmp (darwin_macosx_version_min, "10.5") >= 0)
+        flag_stack_protect = 0;
+    }
+#endif
+  /* LLVM LOCAL end - Don't enable stack protectors by default for Leopard. */
+  /* APPLE LOCAL end stack-protector default 5095227 */
+/* APPLE LOCAL diff confuses me */
 }
 /* APPLE LOCAL begin radar 4985544 */
 bool
@@ -2539,4 +2786,62 @@ darwin_cfstring_type_node (tree type_node)
   return type_node == ccfstring_type_node;
 }
 /* APPLE LOCAL end radar 4985544 */
+
+/* LLVM LOCAL begin radar 6230142 */
+unsigned darwin_llvm_override_target_version(const char *triple, char **new_triple) {
+  int len = 0, pad1 = 0, pad2 = 0, version = 0;
+  char *substr;
+
+  if (!darwin_macosx_version_min)
+    return 0;
+  
+  /* Triple string is expected to look something like 'i386-*-darwin?' or
+     'i386-*-darwin9.5.0'  */
+  substr = strstr(triple, "darwin");
+  if (!substr)
+    return 0;
+  len = substr + 6 - triple;
+
+  /* llvm-gcc doesn't support pre-10.0 systems. */
+  version = strverscmp (darwin_macosx_version_min, "10.0");
+  if (version < 0)
+    return 0;
+
+  /* 10.0 is darwin4. */
+  version += 4;
+
+  /* Darwin version number will be 2 digits for 10.6 and up.  */
+  if (version >= 10)
+    pad1 = 1;
+
+  /* If darwin_macosx_version_min is something like 10.4.9, we need to append
+     the .9 to the new triple. */
+  substr = strchr(darwin_macosx_version_min, '.');
+  if (!substr)
+    return 0;
+  substr = strchr(substr+1, '.');
+  if (substr)
+    pad2 = strlen(substr);
+  
+  *new_triple = ggc_alloc (len+pad1+pad2+1);
+  strncpy (*new_triple, triple, len);
+  if (version >= 10)
+    {
+      (*new_triple)[len] = '1';
+      version -= 10;
+      ++len;
+    }
+  (*new_triple)[len] = '0' + version;
+  if (substr)
+    {
+      int i;
+      for (i = 0; i < pad2; ++i)
+        (*new_triple)[len+1+i] = substr[i];
+      len += pad2;
+    }
+  (*new_triple)[len+1] = '\0';
+  
+  return 1;
+}
+/* LLVM LOCAL end radar 6230142 */
 #include "gt-darwin.h"
