@@ -38,7 +38,12 @@ Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
 #include "langhooks.h"
 #include "real.h"
 
+/* LLVM LOCAL - begin */
+#ifdef ENABLE_LLVM
 #include "llvm.h"
+#endif
+/* LLVM LOCAL - end */
+
 /* Convert EXPR to some pointer or reference type TYPE.
    EXPR must be pointer, reference, integer, enumeral, or literal zero;
    in other cases error is called.  */
@@ -73,13 +78,62 @@ convert_to_pointer (tree type, tree expr)
 			    expr);
       return fold_build1 (CONVERT_EXPR, type, expr);
 
-
+    /* APPLE LOCAL begin blocks (C++ ck) */
+    case BLOCK_POINTER_TYPE:
+        /* APPLE LOCAL begin radar 5809099 */
+        if (objc_is_id (type)
+	    || (TREE_CODE (type) == POINTER_TYPE && VOID_TYPE_P (TREE_TYPE (type))))
+        /* APPLE LOCAL end radar 5809099 */
+          return fold_build1 (NOP_EXPR, type, expr);
+    /* APPLE LOCAL end blocks (C++ ck) */
     default:
       error ("cannot convert to a pointer type");
       return convert_to_pointer (type, integer_zero_node);
     }
 }
 
+/* APPLE LOCAL begin blocks (C++ ck) */
+tree
+convert_to_block_pointer (tree type, tree expr)
+{
+  if (TREE_TYPE (expr) == type)
+    return expr;
+
+  if (integer_zerop (expr))
+    {
+      tree t = build_int_cst (type, 0);
+      if (TREE_OVERFLOW (expr) || TREE_CONSTANT_OVERFLOW (expr))
+	t = force_fit_type (t, 0, TREE_OVERFLOW (expr),
+			    TREE_CONSTANT_OVERFLOW (expr));
+      return t;
+    }
+
+  switch (TREE_CODE (TREE_TYPE (expr)))
+    {
+    case BLOCK_POINTER_TYPE:
+      return fold_build1 (NOP_EXPR, type, expr);
+      
+    case INTEGER_TYPE:
+      if (TYPE_PRECISION (TREE_TYPE (expr)) != POINTER_SIZE)
+	expr = fold_build1 (NOP_EXPR,
+                            lang_hooks.types.type_for_size (POINTER_SIZE, 0),
+			    expr);
+      return fold_build1 (CONVERT_EXPR, type, expr);
+
+    case POINTER_TYPE:
+      /* APPLE LOCAL radar 5809099 */
+      if (objc_is_id (TREE_TYPE (expr)) || VOID_TYPE_P (TREE_TYPE (TREE_TYPE (expr))))
+        return build1 (NOP_EXPR, type, expr);
+      /* fall thru */
+
+    default:
+      error ("cannot convert to a block pointer type");
+      return convert_to_block_pointer (type, integer_zero_node);
+    }
+}
+
+/* APPLE LOCAL end blocks (C++ ck) */
+
 /* Avoid any floating point extensions from EXP.  */
 tree
 strip_float_extensions (tree exp)
@@ -397,9 +451,9 @@ convert_to_integer (tree type, tree expr)
 	    break;
 	  /* LLVM LOCAL begin */
 	  /* FIXME: l-functions should be supported sometimes */
-	  #ifdef ENABLE_LLVM
+#ifdef ENABLE_LLVM
 	  break;
-	  #endif
+#endif
 	  /* LLVM LOCAL end */
 	  if (outprec < TYPE_PRECISION (long_integer_type_node)
 	      || (outprec == TYPE_PRECISION (long_integer_type_node)
@@ -415,9 +469,10 @@ convert_to_integer (tree type, tree expr)
 	  if (!TARGET_C99_FUNCTIONS)
 	    break;
 	  /* LLVM LOCAL begin */
-	  #ifdef ENABLE_LLVM
+#ifdef ENABLE_LLVM
 	  break;
-	  #endif
+#endif
+	  /* LLVM LOCAL end */
 	  if (outprec < TYPE_PRECISION (long_integer_type_node)
 	      || (outprec == TYPE_PRECISION (long_integer_type_node)
 		  && !TYPE_UNSIGNED (type)))
@@ -474,6 +529,8 @@ convert_to_integer (tree type, tree expr)
     {
     case POINTER_TYPE:
     case REFERENCE_TYPE:
+    /* APPLE LOCAL radar 6035389 */
+    case BLOCK_POINTER_TYPE:
       if (integer_zerop (expr))
 	return build_int_cst (type, 0);
 
