@@ -733,7 +733,10 @@ extern enum machine_mode
 mode_for_extraction (enum extraction_pattern, int);
 
 extern rtx store_bit_field (rtx, unsigned HOST_WIDE_INT,
-			    unsigned HOST_WIDE_INT, enum machine_mode, rtx);
+			    /* APPLE LOCAL begin 6020402 */
+			    unsigned HOST_WIDE_INT, enum machine_mode, rtx,
+			    tree);
+			    /* APPLE LOCAL end 6020402 */
 extern rtx extract_bit_field (rtx, unsigned HOST_WIDE_INT,
 			      unsigned HOST_WIDE_INT, int, rtx,
 			      enum machine_mode, enum machine_mode);
@@ -752,7 +755,12 @@ extern void init_all_optabs (void);
 extern rtx init_one_libfunc (const char *);
 /* LLVM LOCAL begin */
 /* Call this to initialize an optab function tree.  */
-extern tree llvm_init_one_libfunc (const char *);
+extern tree llvm_init_one_libfunc_impl (const char *);
+#ifdef ENABLE_LLVM
+#define llvm_init_one_libfunc	llvm_init_one_libfunc_impl
+#else
+#define llvm_init_one_libfunc	init_one_libfunc
+#endif
 /* LLVM LOCAL end */
 
 extern int vector_mode_valid_p (enum machine_mode);
