@@ -41,6 +41,7 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 extern "C" {
 #include "langhooks.h"
 #include "toplev.h"
+#include "flags.h"
 #include "tree.h"
 #include "version.h"
 #include "function.h"
@@ -209,7 +210,6 @@ DebugInfo::DebugInfo(Module *m)
 , PrevLineNo(0)
 , PrevBB(NULL)
 , CurrentGCCLexicalBlock(NULL)
-, FwdTypeCount(0)
 , RegionStack()
 {}
 
@@ -268,14 +268,15 @@ void DebugInfo::push_regions(tree desired, tree grand) {
   // grand, but it should be fixed so nothing weird happens if they're
   // equal.
   llvm::DIDescriptor D = findRegion(desired);
-  RegionStack.push_back(D.getNode());
+  llvm::MDNode *DN = D;
+  RegionStack.push_back(DN);
 }
 
 // Pop the current region/lexical-block back to 'grand', then push
 // regions to arrive at 'desired'.  This was inspired (cribbed from)
 // by GCC's cfglayout.c:change_scope().
 void DebugInfo::change_regions(tree desired, tree grand) {
-  tree current_lexical_block = getCurrentLexicalBlock();
+  tree current_lexical_block = getCurrentLexicalBlock(), t;
   // FIXME: change_regions is currently never called with desired ==
   // grand, but it should be fixed so nothing weird happens if they're
   // equal.
@@ -289,26 +290,25 @@ void DebugInfo::change_regions(tree desired, tree grand) {
   setCurrentLexicalBlock(desired);
 }
 
-/// EmitFunctionStart - Constructs the debug code for entering a function -
-/// "llvm.dbg.func.start."
-void DebugInfo::EmitFunctionStart(tree FnDecl, Function *Fn,
-                                  BasicBlock *CurBB) {
-  setCurrentLexicalBlock(FnDecl);
-
+/// CreateSubprogramFromFnDecl - Constructs the debug code for
+/// entering a function - "llvm.dbg.func.start."
+DISubprogram DebugInfo::CreateSubprogramFromFnDecl(tree FnDecl) {
+  DISubprogram SPDecl;
+  bool SPDeclIsSet = false;
+  // True if we're currently generating LLVM for this function.
+  bool definition = llvm_set_decl_p(FnDecl);
   DIType FNType = getOrCreateType(TREE_TYPE(FnDecl));
 
   std::map<tree_node *, WeakVH >::iterator I = SPCache.find(FnDecl);
   if (I != SPCache.end()) {
-    DISubprogram SPDecl(cast<MDNode>(I->second));
-    DISubprogram SP = 
-      DebugFactory.CreateSubprogramDefinition(SPDecl);
-    SPDecl.getNode()->replaceAllUsesWith(SP.getNode());
-
-    // Push function on region stack.
-    RegionStack.push_back(WeakVH(SP.getNode()));
-    RegionMap[FnDecl] = WeakVH(SP.getNode());
-    return;
-  } 
+    SPDecl = DISubprogram(cast<MDNode>(I->second));
+    SPDeclIsSet = true;
+    // If we've already created the defining instance, OR this
+    // invocation won't create the defining instance, return what we
+    // already have.
+    if (SPDecl.isDefinition() || !definition)
+      return SPDecl;
+  }
 
   bool ArtificialFnWithAbstractOrigin = false;
   // If this artificial function has abstract origin then put this function
@@ -319,29 +319,37 @@ void DebugInfo::EmitFunctionStart(tree FnDecl, Function *Fn,
     ArtificialFnWithAbstractOrigin = true;
 
   DIDescriptor SPContext = ArtificialFnWithAbstractOrigin ?
-    getOrCreateCompileUnit(main_input_filename) :
+    getOrCreateFile(main_input_filename) :
     findRegion (DECL_CONTEXT(FnDecl));
 
+  // Declare block_invoke functions at file scope for GDB.
+  if (BLOCK_SYNTHESIZED_FUNC(FnDecl))
+    SPContext = findRegion(NULL_TREE);
+
   // Creating context may have triggered creation of this SP descriptor. So
   // check the cache again.
-  I = SPCache.find(FnDecl);
-  if (I != SPCache.end()) {
-    DISubprogram SPDecl(cast<MDNode>(I->second));
-    DISubprogram SP = 
-      DebugFactory.CreateSubprogramDefinition(SPDecl);
-    SPDecl.getNode()->replaceAllUsesWith(SP.getNode());
-
-    // Push function on region stack.
-    RegionStack.push_back(WeakVH(SP.getNode()));
-    RegionMap[FnDecl] = WeakVH(SP.getNode());
-    return;
-  } 
+  if (!SPDeclIsSet) {
+    I = SPCache.find(FnDecl);
+    if (I != SPCache.end()) {
+      SPDecl = DISubprogram(cast<MDNode>(I->second));
+      DISubprogram SP = 
+        DebugFactory.CreateSubprogramDefinition(SPDecl);
+      SPDecl->replaceAllUsesWith(SP);
 
+      // Push function on region stack.
+      RegionStack.push_back(WeakVH(SP));
+      RegionMap[FnDecl] = WeakVH(SP);
+      return SP;
+    }
+  }
   // Gather location information.
   expanded_location Loc = GetNodeLocation(FnDecl, false);
-  StringRef LinkageName = getLinkageName(FnDecl);
+  // If the name isn't public, omit the linkage name.  Adding a
+  // linkage name to a class method can confuse GDB.
+  StringRef LinkageName = TREE_PUBLIC(FnDecl) ?
+    getLinkageName(FnDecl) : StringRef();
 
-  unsigned lineno = CurLineNo;
+  unsigned lineno = LOCATION_LINE(Loc);
   if (isCopyOrDestroyHelper(FnDecl))
     lineno = 0;
 
@@ -356,23 +364,40 @@ void DebugInfo::EmitFunctionStart(tree FnDecl, Function *Fn,
   }
 
   StringRef FnName = getFunctionName(FnDecl);
-
+  // If the Function * hasn't been created yet, use a bogus value for
+  // the debug internal linkage bit.
+  bool hasInternalLinkage = true;
+  Function *Fn = 0;
+  if (GET_DECL_LLVM_INDEX(FnDecl)) {
+    Fn = cast<Function>DECL_LLVM(FnDecl);
+    if (Fn)
+      hasInternalLinkage = Fn->hasInternalLinkage();
+  }
   DISubprogram SP = 
     DebugFactory.CreateSubprogram(SPContext,
                                   FnName, FnName,
                                   LinkageName,
-                                  getOrCreateCompileUnit(Loc.file), lineno,
+                                  getOrCreateFile(Loc.file), lineno,
                                   FNType,
-                                  Fn->hasInternalLinkage(),
-                                  true /*definition*/,
-                                  Virtuality, VIndex, ContainingType);
-                          
+                                  hasInternalLinkage,
+                                  definition,
+                                  Virtuality, VIndex, ContainingType,
+                                  DECL_ARTIFICIAL (FnDecl), optimize,
+                                  Fn);
 
-  SPCache[FnDecl] = WeakVH(SP.getNode());
+  SPCache[FnDecl] = WeakVH(SP);
+  RegionMap[FnDecl] = WeakVH(SP);
+  if (SPDeclIsSet && SPDecl != SP)
+    SPDecl->replaceAllUsesWith(SP);
+  return SP;
+}
 
+/// EmitFunctionStart - Constructs the debug code for entering a function.
+void DebugInfo::EmitFunctionStart(tree FnDecl) {
+  setCurrentLexicalBlock(FnDecl);
+  DISubprogram SP = CreateSubprogramFromFnDecl(FnDecl);
   // Push function on region stack.
-  RegionStack.push_back(WeakVH(SP.getNode()));
-  RegionMap[FnDecl] = WeakVH(SP.getNode());
+  RegionStack.push_back(WeakVH(SP));
 }
 
 /// getOrCreateNameSpace - Get name space descriptor for the tree node.
@@ -385,16 +410,98 @@ DINameSpace DebugInfo::getOrCreateNameSpace(tree Node, DIDescriptor Context) {
   expanded_location Loc = GetNodeLocation(Node, false);
   DINameSpace DNS =
     DebugFactory.CreateNameSpace(Context, GetNodeName(Node),
-                                 getOrCreateCompileUnit(Loc.file), Loc.line);
+                                 getOrCreateFile(Loc.file), Loc.line);
 
-  NameSpaceCache[Node] = WeakVH(DNS.getNode());
+  NameSpaceCache[Node] = WeakVH(DNS);
   return DNS;
 }
 
-/// findRegion - Find tree_node N's region.
-DIDescriptor DebugInfo::findRegion(tree Node) {
-  if (Node == NULL_TREE)
-    return getOrCreateCompileUnit(main_input_filename);
+/// variablesDeclaredAtThisLevel - yield true if this block, or any
+/// sibling thereof, declares any variables or types.  Expectes a
+/// BLOCK or FUNCTION_DECL tree.
+// GDB Kludge
+static bool variablesDeclaredAtThisLevel(tree blk) {
+  if (TREE_CODE(blk) == FUNCTION_DECL)
+    return true;
+  assert(TREE_CODE(blk) == BLOCK && "expected FUNCTION_DECL or BLOCK tree");
+  tree supercontext = BLOCK_SUPERCONTEXT(blk);
+  if (!supercontext)	// External scope.  Pretend it's the same as module scope.
+    return false;
+  tree step = TREE_CODE(supercontext) == BLOCK ?
+    BLOCK_SUBBLOCKS(supercontext) : DECL_INITIAL(supercontext);
+  for ( ; step && TREE_CODE(step) == BLOCK; step = TREE_CHAIN(step))
+    if (BLOCK_VARS(step))
+      return true;
+  return false;
+}
+
+/// supercontextWithDecls - walk up the BLOCKS tree, return an
+/// ancestor BLOCK that has vars, or sibling BLOCKs that do.  May
+/// return the FUNCTION_DECL.
+// GDB Kludge
+static tree supercontextWithDecls(tree blk) {
+  assert(TREE_CODE(blk) == BLOCK && "expected BLOCK");
+  tree step = BLOCK_SUPERCONTEXT(blk);
+  if (!step)    // No parent BLOCKs to search.
+    return blk;
+  while (step && TREE_CODE(step) == BLOCK && BLOCK_SUPERCONTEXT(step) &&
+         !variablesDeclaredAtThisLevel(step))
+    step = BLOCK_SUPERCONTEXT(step);
+  return step;
+}
+
+/// nonemptySibling - true if the given BLOCK has a sibling that is
+/// not empty (declares types and/or vars).
+// GDB Kludge
+static bool nonemptySibling(tree blk) {
+  if (TREE_CODE(blk) == FUNCTION_DECL)
+    return true;
+  assert(TREE_CODE(blk) == BLOCK && "expected FUNCTION_DECL or BLOCK tree");
+  tree supercontext = BLOCK_SUPERCONTEXT(blk);
+  if (!supercontext)	// External scope.  Pretend it's the same as module scope.
+    return false;
+  tree step = TREE_CODE(supercontext) == BLOCK ?
+    BLOCK_SUBBLOCKS(supercontext) : DECL_INITIAL(supercontext);
+  for ( ; step; step = TREE_CHAIN(step))
+    if (step != blk && BLOCK_VARS(step))
+      return true;
+  return false;
+}
+
+/// findRegion - Find the region (context) of a GCC tree.
+DIDescriptor DebugInfo::findRegion(tree exp) {
+  if (exp == NULL_TREE)
+    return getOrCreateFile(main_input_filename);
+
+  tree Node = exp;
+  location_t *p_locus = 0;
+  tree_code code = TREE_CODE(exp);
+  enum tree_code_class tree_cc = TREE_CODE_CLASS(code);
+  switch (tree_cc) {
+  case tcc_declaration:  /* A decl node */
+    p_locus = &DECL_SOURCE_LOCATION(exp);
+    break;
+
+  case tcc_expression:  /* an expression */
+  case tcc_comparison:  /* a comparison expression */
+  case tcc_unary:  /* a unary arithmetic expression */
+  case tcc_binary:  /* a binary arithmetic expression */
+    Node = TREE_BLOCK(exp);
+    p_locus = EXPR_LOCUS(exp);
+    break;
+
+  case tcc_exceptional:
+    switch (code) {
+    case BLOCK:
+      p_locus = &BLOCK_SOURCE_LOCATION(Node);
+      break;
+    default:
+      gcc_unreachable ();
+    }
+    break;
+  default:
+    break;
+  }
 
   std::map<tree_node *, WeakVH>::iterator I = RegionMap.find(Node);
   if (I != RegionMap.end())
@@ -403,30 +510,47 @@ DIDescriptor DebugInfo::findRegion(tree Node) {
 
   if (TYPE_P (Node)) {
     DIType Ty = getOrCreateType(Node);
-    return DIDescriptor(Ty.getNode());
+    return DIDescriptor(Ty);
   } else if (DECL_P (Node)) {
-    if (TREE_CODE (Node) == NAMESPACE_DECL) {
+    switch (TREE_CODE(Node)) {
+    default:
+      /// What kind of DECL is this?
+      return findRegion (DECL_CONTEXT (Node));
+    case NAMESPACE_DECL: {
       DIDescriptor NSContext = findRegion(DECL_CONTEXT(Node));
       DINameSpace NS = getOrCreateNameSpace(Node, NSContext);
-      return DIDescriptor(NS.getNode());
+      return DIDescriptor(NS);
+    }
+    case FUNCTION_DECL: {
+      DISubprogram SP = CreateSubprogramFromFnDecl(Node);
+      return SP;
+    }
     }
-    return findRegion (DECL_CONTEXT (Node));
   } else if (TREE_CODE(Node) == BLOCK) {
-    // TREE_BLOCK is GCC's lexical block.
-    // Recursively create all necessary contexts:
+    // Recursively establish ancestor scopes.
     DIDescriptor context = findRegion(BLOCK_SUPERCONTEXT(Node));
+    // If we don't have a location, use the last-seen info.
+    unsigned int line;
+    const char *fullpath;
+    if (LOCATION_FILE(*p_locus) == (char*)0) {
+      fullpath = CurFullPath;
+      line = CurLineNo;
+    } else {
+      fullpath = LOCATION_FILE(*p_locus);
+      line = LOCATION_LINE(*p_locus);
+    }
+    DIFile F(getOrCreateFile(fullpath));
     DILexicalBlock lexical_block = 
-      DebugFactory.CreateLexicalBlock(context, CurLineNo);
-    RegionMap[Node] = WeakVH(lexical_block.getNode());
+      DebugFactory.CreateLexicalBlock(context, F, line, 0U);
+    RegionMap[Node] = WeakVH(lexical_block);
     return DIDescriptor(lexical_block);
   }
 
   // Otherwise main compile unit covers everything.
-  return getOrCreateCompileUnit(main_input_filename);
+  return getOrCreateFile(main_input_filename);
 }
 
-/// EmitFunctionEnd - Constructs the debug code for exiting a declarative
-/// region - "llvm.dbg.region.end."
+/// EmitFunctionEnd - Pop the region stack and reset current lexical block.
 void DebugInfo::EmitFunctionEnd(BasicBlock *CurBB, bool EndFunction) {
   assert(!RegionStack.empty() && "Region stack mismatch, stack empty!");
   RegionStack.pop_back();
@@ -441,14 +565,9 @@ void DebugInfo::EmitFunctionEnd(BasicBlock *CurBB, bool EndFunction) {
 }
 
 /// EmitDeclare - Constructs the debug code for allocation of a new variable.
-/// region - "llvm.dbg.declare."
 void DebugInfo::EmitDeclare(tree decl, unsigned Tag, const char *Name,
                             tree type, Value *AI, LLVMBuilder &Builder) {
 
-  // Do not emit variable declaration info, for now.
-  if (optimize)
-    return;
-
   // Ignore compiler generated temporaries.
   if (DECL_IGNORED_P(decl))
     return;
@@ -460,22 +579,22 @@ void DebugInfo::EmitDeclare(tree decl, unsigned Tag, const char *Name,
   // Construct variable.
   DIScope VarScope = DIScope(cast<MDNode>(RegionStack.back()));
   DIType Ty = getOrCreateType(type);
+  if (!Ty && TREE_CODE(type) == OFFSET_TYPE)
+    Ty = createPointerType(TREE_TYPE(type));
   if (DECL_ARTIFICIAL (decl))
       Ty = DebugFactory.CreateArtificialType(Ty);
+  // If type info is not available then do not emit debug info for this var.
+  if (!Ty)
+    return;
   llvm::DIVariable D =
     DebugFactory.CreateVariable(Tag, VarScope,
-                                Name, getOrCreateCompileUnit(Loc.file),
-                                Loc.line, Ty);
+                                Name, getOrCreateFile(Loc.file),
+                                Loc.line, Ty, optimize);
 
-  // Insert an llvm.dbg.declare into the current block.
-  Instruction *Call = DebugFactory.InsertDeclare(AI, D, 
-                                                 Builder.GetInsertBlock());
-
-  llvm::DILocation DO(NULL);
-  llvm::DILocation DL = 
-    DebugFactory.CreateLocation(CurLineNo, 0 /* column */, VarScope, DO);
+  Instruction *Call = 
+    DebugFactory.InsertDeclare(AI, D, Builder.GetInsertBlock());
   
-  Call->setMetadata("dbg", DL.getNode());
+  Call->setDebugLoc(DebugLoc::get(Loc.line, 0, VarScope));
 }
 
 
@@ -501,8 +620,7 @@ bool isPartOfAppleBlockPrologue (unsigned lineno) {
   return false;
 }
 
-/// EmitStopPoint - Emit a call to llvm.dbg.stoppoint to indicate a change of 
-/// source line - "llvm.dbg.stoppoint."  Now enabled at -O.
+/// EmitStopPoint - Set current source location. 
 void DebugInfo::EmitStopPoint(Function *Fn, BasicBlock *CurBB,
                               LLVMBuilder &Builder) {
   // Don't bother if things are the same as last time.
@@ -524,19 +642,15 @@ void DebugInfo::EmitStopPoint(Function *Fn, BasicBlock *CurBB,
     
     if (RegionStack.empty())
       return;
-    llvm::DIDescriptor DR(cast<MDNode>(RegionStack.back()));
-    llvm::DIScope DS = llvm::DIScope(DR.getNode());
-    llvm::DILocation DO(NULL);
-    llvm::DILocation DL = 
-      DebugFactory.CreateLocation(CurLineNo, 0 /* column */, DS, DO);
-    Builder.SetCurrentDebugLocation(DL.getNode());
+    MDNode *Scope = cast<MDNode>(RegionStack.back());
+    Builder.SetCurrentDebugLocation(DebugLoc::get(CurLineNo,0/*col*/,Scope));
   }
 }
 
 /// EmitGlobalVariable - Emit information about a global variable.
 ///
 void DebugInfo::EmitGlobalVariable(GlobalVariable *GV, tree decl) {
-  if (DECL_ARTIFICIAL(decl))
+  if (DECL_ARTIFICIAL(decl) || DECL_IGNORED_P(decl))
     return;
   // Gather location information.
   expanded_location Loc = expand_location(DECL_SOURCE_LOCATION(decl));
@@ -551,9 +665,14 @@ void DebugInfo::EmitGlobalVariable(GlobalVariable *GV, tree decl) {
   if (DECL_CONTEXT (decl))
     if (TREE_CODE (DECL_CONTEXT (decl)) != FUNCTION_DECL)
       LinkageName = GV->getName();
-  DebugFactory.CreateGlobalVariable(findRegion(DECL_CONTEXT(decl)),
+  DIDescriptor Context;
+  if (DECL_CONTEXT(decl)) 
+    Context = findRegion(DECL_CONTEXT(decl));
+  else
+    Context = getOrCreateFile(Loc.file);
+  DebugFactory.CreateGlobalVariable(Context,
                                     DispName, DispName, LinkageName,
-                                    getOrCreateCompileUnit(Loc.file), Loc.line,
+                                    getOrCreateFile(Loc.file), Loc.line,
                                     TyD, GV->hasInternalLinkage(),
                                     true/*definition*/, GV);
 }
@@ -599,12 +718,37 @@ DIType DebugInfo::createBasicType(tree type) {
   }
   }
 
-  return 
-    DebugFactory.CreateBasicType(getOrCreateCompileUnit(main_input_filename),
+  DIBasicType BTy =  
+    DebugFactory.CreateBasicType(getOrCreateFile(main_input_filename),
                                  TypeName, 
-                                 getOrCreateCompileUnit(main_input_filename),
+                                 getOrCreateFile(main_input_filename),
                                  0, Size, Align,
                                  0, 0, Encoding);
+
+  if (TheDebugInfo && flag_pch_file) {
+    NamedMDNode *NMD = TheModule->getOrInsertNamedMetadata("llvm.dbg.pch.bt");
+    NMD->addOperand(BTy);
+  }
+
+  return BTy;
+}
+
+/// replaceBasicTypesFromPCH - Replace basic type debug info received
+/// from PCH file.
+void DebugInfo::replaceBasicTypesFromPCH() {
+  NamedMDNode *NMD = TheModule->getOrInsertNamedMetadata("llvm.dbg.pch.bt");
+  for (int i = 0, e = NMD->getNumOperands(); i != e; ++i) {
+    DIBasicType HeaderBTy(NMD->getOperand(i));
+    MDNode *NewBTy = NULL;
+    NewBTy = DebugFactory.CreateBasicType(getOrCreateFile(main_input_filename),
+                                          HeaderBTy.getName(),
+                                          getOrCreateFile(main_input_filename),
+                                          0, HeaderBTy.getSizeInBits(), 
+                                          HeaderBTy.getAlignInBits(),
+                                          0, 0, HeaderBTy.getEncoding());
+    MDNode *HBTyNode = HeaderBTy;
+    HBTyNode->replaceAllUsesWith(NewBTy);
+  }
 }
 
 /// isArtificialArgumentType - Return true if arg_type represents artificial,
@@ -627,20 +771,13 @@ DIType DebugInfo::createMethodType(tree type) {
 
   // Create a  place holder type first. The may be used as a context
   // for the argument types.
-  char *FwdTypeName = (char *)alloca(65);
-  sprintf(FwdTypeName, "fwd.type.%d", FwdTypeCount++);
-  llvm::DIType FwdType = 
-    DebugFactory.CreateCompositeType(llvm::dwarf::DW_TAG_subroutine_type,
-                                     getOrCreateCompileUnit(NULL), 
-                                     FwdTypeName,
-                                     getOrCreateCompileUnit(NULL), 
-                                     0, 0, 0, 0, 0,
-                                     llvm::DIType(), llvm::DIArray());
-  llvm::TrackingVH<llvm::MDNode> FwdTypeNode = FwdType.getNode();
-  TypeCache[type] = WeakVH(FwdType.getNode());
+  llvm::DIType FwdType = DebugFactory.CreateTemporaryType();
+  llvm::MDNode *FTN = FwdType;
+  llvm::TrackingVH<llvm::MDNode> FwdTypeNode = FTN;
+  TypeCache[type] = WeakVH(FwdType);
   // Push the struct on region stack.
-  RegionStack.push_back(WeakVH(FwdType.getNode()));
-  RegionMap[type] = WeakVH(FwdType.getNode());
+  RegionStack.push_back(WeakVH(FwdType));
+  RegionMap[type] = WeakVH(FwdType);
   
   llvm::SmallVector<llvm::DIDescriptor, 16> EltTys;
   
@@ -674,13 +811,13 @@ DIType DebugInfo::createMethodType(tree type) {
     DebugFactory.CreateCompositeType(llvm::dwarf::DW_TAG_subroutine_type,
                                      findRegion(TYPE_CONTEXT(type)), 
                                      StringRef(),
-                                     getOrCreateCompileUnit(NULL), 
+                                     getOrCreateFile(main_input_filename),
                                      0, 0, 0, 0, 0,
                                      llvm::DIType(), EltTypeArray);
 
   // Now that we have a real decl for the struct, replace anything using the
   // old decl with the new one.  This will recursively update the debug info.
-  llvm::DIDerivedType(FwdTypeNode).replaceAllUsesWith(RealType);
+  llvm::DIType(FwdTypeNode).replaceAllUsesWith(RealType);
 
   return RealType;
 }
@@ -691,10 +828,8 @@ DIType DebugInfo::createPointerType(tree type) {
   DIType FromTy = getOrCreateType(TREE_TYPE(type));
   // type* and type&
   // FIXME: Should BLOCK_POINTER_TYP have its own DW_TAG?
-  unsigned Tag = (TREE_CODE(type) == POINTER_TYPE ||
-                  TREE_CODE(type) == BLOCK_POINTER_TYPE) ?
-    DW_TAG_pointer_type :
-    DW_TAG_reference_type;
+  unsigned Tag = TREE_CODE(type) == REFERENCE_TYPE ?
+    DW_TAG_reference_type: DW_TAG_pointer_type;
   unsigned Flags = 0;
   if (type_is_block_byref_struct(type))
     Flags |= llvm::DIType::FlagBlockByrefStruct;
@@ -706,23 +841,23 @@ DIType DebugInfo::createPointerType(tree type) {
       DIType Ty = 
         DebugFactory.CreateDerivedType(Tag, findRegion(DECL_CONTEXT(TyName)),
                                        GetNodeName(TyName), 
-                                       getOrCreateCompileUnit(TypeNameLoc.file),
+                                       getOrCreateFile(TypeNameLoc.file),
                                        TypeNameLoc.line,
                                        0 /*size*/,
                                        0 /*align*/,
                                        0 /*offset */, 
                                        0 /*flags*/, 
                                        FromTy);
-      TypeCache[TyName] = WeakVH(Ty.getNode());
+      TypeCache[TyName] = WeakVH(Ty);
       return Ty;
     }
   
   StringRef PName = FromTy.getName();
   DIType PTy = 
-    DebugFactory.CreateDerivedType(Tag, findRegion(TYPE_CONTEXT(type)), 
+    DebugFactory.CreateDerivedType(Tag, findRegion(TYPE_CONTEXT(type)),
                                    Tag == DW_TAG_pointer_type ? 
                                    StringRef() : PName,
-                                   getOrCreateCompileUnit(NULL), 
+                                   getOrCreateFile(main_input_filename),
                                    0 /*line no*/, 
                                    NodeSizeInBits(type),
                                    NodeAlignInBits(type),
@@ -784,7 +919,7 @@ DIType DebugInfo::createArrayType(tree type) {
   return DebugFactory.CreateCompositeType(llvm::dwarf::DW_TAG_array_type,
                                           findRegion(TYPE_CONTEXT(type)), 
                                           StringRef(),
-                                          getOrCreateCompileUnit(Loc.file), 0, 
+                                          getOrCreateFile(Loc.file), 0, 
                                           NodeSizeInBits(type), 
                                           NodeAlignInBits(type), 0, 0,
                                           getOrCreateType(EltTy),
@@ -816,7 +951,7 @@ DIType DebugInfo::createEnumType(tree type) {
   return DebugFactory.CreateCompositeType(llvm::dwarf::DW_TAG_enumeration_type,
                                           findRegion(TYPE_CONTEXT(type)), 
                                           GetNodeName(type), 
-                                          getOrCreateCompileUnit(Loc.file), 
+                                          getOrCreateFile(Loc.file), 
                                           Loc.line,
                                           NodeSizeInBits(type), 
                                           NodeAlignInBits(type), 0, 0,
@@ -834,8 +969,7 @@ DIType DebugInfo::createStructType(tree type) {
   if (TYPE_LANG_SPECIFIC (type)
       && lang_hooks.types.is_runtime_specific_type (type))
     {
-      DICompileUnit CU = getOrCreateCompileUnit(main_input_filename);
-      unsigned CULang = CU.getLanguage();
+      unsigned CULang = TheCU.getLanguage();
       switch (CULang) {
       case DW_LANG_ObjC_plus_plus :
         RunTimeLang = DW_LANG_ObjC_plus_plus;
@@ -859,19 +993,6 @@ DIType DebugInfo::createStructType(tree type) {
   // recursive) and replace all  uses of the forward declaration with the 
   // final definition. 
   expanded_location Loc = GetNodeLocation(TREE_CHAIN(type), false);
-  // FIXME: findRegion() is not able to find context all the time. This
-  // means when type names in different context match then FwdDecl is
-  // reused because MDNodes are uniqued. To avoid this, use type context
-  /// also while creating FwdDecl for now.
-  std::string FwdName;
-  if (TYPE_CONTEXT(type)) {
-    StringRef TypeContextName = GetNodeName(TYPE_CONTEXT(type));
-    if (!TypeContextName.empty())
-      FwdName = TypeContextName;
-  }
-  StringRef TypeName = GetNodeName(type);
-  if (!TypeName.empty())
-    FwdName = FwdName + TypeName.data();
   unsigned SFlags = 0;
   if (TYPE_BLOCK_IMPL_STRUCT(type))
     SFlags |= llvm::DIType::FlagAppleBlock;
@@ -886,27 +1007,31 @@ DIType DebugInfo::createStructType(tree type) {
     if (MDNode *TN = dyn_cast_or_null<MDNode>(I->second))
       return DIType(TN);
   
-  llvm::DICompositeType FwdDecl =
-    DebugFactory.CreateCompositeType(Tag, 
-                                     TyContext,
-                                     FwdName.c_str(),
-                                     getOrCreateCompileUnit(Loc.file), 
-                                     Loc.line, 
-                                     0, 0, 0, SFlags | llvm::DIType::FlagFwdDecl,
-                                     llvm::DIType(), llvm::DIArray(),
-                                     RunTimeLang);
-  
   // forward declaration, 
-  if (TYPE_SIZE(type) == 0) 
+  if (TYPE_SIZE(type) == 0) {
+    llvm::DICompositeType FwdDecl =
+      DebugFactory.CreateCompositeType(Tag, 
+                                       TyContext,
+                                       GetNodeName(type),
+                                       getOrCreateFile(Loc.file), 
+                                       Loc.line, 
+                                       0, 0, 0,
+                                       SFlags | llvm::DIType::FlagFwdDecl,
+                                       llvm::DIType(), llvm::DIArray(),
+                                       RunTimeLang);
     return FwdDecl;
+  }
+  
+  llvm::DIType FwdDecl = DebugFactory.CreateTemporaryType();
   
   // Insert into the TypeCache so that recursive uses will find it.
-  llvm::TrackingVH<llvm::MDNode> FwdDeclNode = FwdDecl.getNode();
-  TypeCache[type] =  WeakVH(FwdDecl.getNode());
+  llvm::MDNode *FDN = FwdDecl;
+  llvm::TrackingVH<llvm::MDNode> FwdDeclNode = FDN;
+  TypeCache[type] =  WeakVH(FwdDecl);
 
   // Push the struct on region stack.
-  RegionStack.push_back(WeakVH(FwdDecl.getNode()));
-  RegionMap[type] = WeakVH(FwdDecl.getNode());
+  RegionStack.push_back(WeakVH(FwdDecl));
+  RegionMap[type] = WeakVH(FwdDecl);
   
   // Convert all the elements.
   llvm::SmallVector<llvm::DIDescriptor, 16> EltTys;
@@ -940,7 +1065,7 @@ DIType DebugInfo::createStructType(tree type) {
       DIType DTy = 
         DebugFactory.CreateDerivedType(DW_TAG_inheritance, 
                                        findRegion(TYPE_CONTEXT(type)), StringRef(),
-                                       llvm::DICompileUnit(), 0,0,0, 
+                                       llvm::DIFile(), 0,0,0, 
                                        Offset,
                                        BFlags, BaseClass);
       EltTys.push_back(DTy);
@@ -986,7 +1111,7 @@ DIType DebugInfo::createStructType(tree type) {
       DebugFactory.CreateDerivedType(DW_TAG_member, 
                                      findRegion(DECL_CONTEXT(Member)),
                                      MemberName, 
-                                     getOrCreateCompileUnit(MemLoc.file),
+                                     getOrCreateFile(MemLoc.file),
                                      MemLoc.line, NodeSizeInBits(Member),
                                      NodeAlignInBits(FieldNodeType),
                                      int_bit_position(Member), 
@@ -1025,12 +1150,12 @@ DIType DebugInfo::createStructType(tree type) {
         DebugFactory.CreateSubprogram(findRegion(DECL_CONTEXT(Member)), 
                                       MemberName, MemberName,
                                       LinkageName, 
-                                      getOrCreateCompileUnit(MemLoc.file),
+                                      getOrCreateFile(MemLoc.file),
                                       MemLoc.line, SPTy, false, false,
                                       Virtuality, VIndex, ContainingType,
-                                      DECL_ARTIFICIAL (Member));
+                                      DECL_ARTIFICIAL (Member), optimize);
       EltTys.push_back(SP);
-      SPCache[Member] = WeakVH(SP.getNode());
+      SPCache[Member] = WeakVH(SP);
     }
   }
   
@@ -1050,16 +1175,16 @@ DIType DebugInfo::createStructType(tree type) {
   llvm::DICompositeType RealDecl =
     DebugFactory.CreateCompositeType(Tag, findRegion(TYPE_CONTEXT(type)),
                                      GetNodeName(type),
-                                     getOrCreateCompileUnit(Loc.file),
+                                     getOrCreateFile(Loc.file),
                                      Loc.line, 
                                      NodeSizeInBits(type), NodeAlignInBits(type),
                                      0, SFlags, llvm::DIType(), Elements,
-                                     RunTimeLang, ContainingType.getNode());
-  RegionMap[type] = WeakVH(RealDecl.getNode());
+                                     RunTimeLang, ContainingType);
+  RegionMap[type] = WeakVH(RealDecl);
 
   // Now that we have a real decl for the struct, replace anything using the
   // old decl with the new one.  This will recursively update the debug info.
-  llvm::DIDerivedType(FwdDeclNode).replaceAllUsesWith(RealDecl);
+  llvm::DIType(FwdDeclNode).replaceAllUsesWith(RealDecl);
 
   return RealDecl;
 }
@@ -1075,17 +1200,22 @@ DIType DebugInfo::createVariantType(tree type, DIType MainTy) {
           return DIType(cast<MDNode>(M));
     if (TREE_CODE(TyDef) == TYPE_DECL &&  DECL_ORIGINAL_TYPE(TyDef)) {
       expanded_location TypeDefLoc = GetNodeLocation(TyDef);
+      DIDescriptor Context;
+      if (DECL_CONTEXT(TyDef)) 
+        Context = findRegion(DECL_CONTEXT(TyDef));
+      else
+        Context = getOrCreateFile(TypeDefLoc.file);
       Ty = DebugFactory.CreateDerivedType(DW_TAG_typedef, 
-                                          findRegion(DECL_CONTEXT(TyDef)),
+                                          Context,
                                           GetNodeName(TyDef), 
-                                          getOrCreateCompileUnit(TypeDefLoc.file),
+                                          getOrCreateFile(TypeDefLoc.file),
                                           TypeDefLoc.line,
                                           0 /*size*/,
                                           0 /*align*/,
                                           0 /*offset */, 
                                           0 /*flags*/, 
                                           MainTy);
-      TypeCache[TyDef] = WeakVH(Ty.getNode());
+      TypeCache[TyDef] = WeakVH(Ty);
       return Ty;
     }
   }
@@ -1094,7 +1224,7 @@ DIType DebugInfo::createVariantType(tree type, DIType MainTy) {
     Ty = DebugFactory.CreateDerivedType(DW_TAG_volatile_type, 
                                         findRegion(TYPE_CONTEXT(type)), 
                                         StringRef(),
-                                        getOrCreateCompileUnit(NULL), 
+                                        getOrCreateFile(main_input_filename),
                                         0 /*line no*/, 
                                         NodeSizeInBits(type),
                                         NodeAlignInBits(type),
@@ -1108,7 +1238,7 @@ DIType DebugInfo::createVariantType(tree type, DIType MainTy) {
     Ty =  DebugFactory.CreateDerivedType(DW_TAG_const_type, 
                                          findRegion(TYPE_CONTEXT(type)), 
                                          StringRef(),
-                                         getOrCreateCompileUnit(NULL), 
+                                         getOrCreateFile(main_input_filename),
                                          0 /*line no*/, 
                                          NodeSizeInBits(type),
                                          NodeAlignInBits(type),
@@ -1117,7 +1247,7 @@ DIType DebugInfo::createVariantType(tree type, DIType MainTy) {
                                          MainTy);
   
   if (TYPE_VOLATILE(type) || TYPE_READONLY(type)) {
-    TypeCache[type] = WeakVH(Ty.getNode());
+    TypeCache[type] = WeakVH(Ty);
     return Ty;
   }
 
@@ -1147,7 +1277,7 @@ DIType DebugInfo::getOrCreateType(tree type) {
     MainTy = getOrCreateType(TYPE_MAIN_VARIANT(type));
 
   DIType Ty = createVariantType(type, MainTy);
-  if (!Ty.isNull())
+  if (Ty.isValid())
     return Ty;
 
   // Work out details of type.
@@ -1179,6 +1309,7 @@ DIType DebugInfo::getOrCreateType(tree type) {
       // gen_type_die(TYPE_OFFSET_BASETYPE(type), context_die);
       // gen_type_die(TREE_TYPE(type), context_die);
       // gen_ptr_to_mbr_type_die(type, context_die);
+      // PR 7104
       break;
     }
 
@@ -1209,7 +1340,7 @@ DIType DebugInfo::getOrCreateType(tree type) {
       Ty = createBasicType(type);
       break;
   }
-  TypeCache[type] = WeakVH(Ty.getNode());
+  TypeCache[type] = WeakVH(Ty);
   return Ty;
 }
 
@@ -1227,9 +1358,9 @@ void DebugInfo::Initialize() {
   // module does not contain any main compile unit then the code generator 
   // will emit multiple compile units in the output object file.
   if (!strcmp (main_input_filename, ""))
-    getOrCreateCompileUnit("<stdin>", true);
+    TheCU = getOrCreateCompileUnit("<stdin>", true);
   else
-    getOrCreateCompileUnit(main_input_filename, true);
+    TheCU = getOrCreateCompileUnit(main_input_filename, true);
 }
 
 /// getOrCreateCompileUnit - Get the compile unit from the cache or 
@@ -1242,10 +1373,6 @@ DICompileUnit DebugInfo::getOrCreateCompileUnit(const char *FullPath,
     else
       FullPath = main_input_filename;
   }
-  std::map<std::string, WeakVH >::iterator I = CUCache.find(FullPath);
-  if (I != CUCache.end())
-    if (Value *M = I->second)
-      return DICompileUnit(cast<MDNode>(M));
 
   // Get source file information.
   std::string Directory;
@@ -1286,13 +1413,27 @@ DICompileUnit DebugInfo::getOrCreateCompileUnit(const char *FullPath,
   unsigned ObjcRunTimeVer = 0;
   if (flag_objc_abi != 0 && flag_objc_abi != -1)
     ObjcRunTimeVer = flag_objc_abi;
-  DICompileUnit NewCU = DebugFactory.CreateCompileUnit(LangTag, FileName.c_str(),
-                                                       Directory.c_str(),
-                                                       version_string, isMain,
-                                                       optimize, Flags,
-                                                       ObjcRunTimeVer);
-  CUCache[FullPath] = WeakVH(NewCU.getNode());
-  return NewCU;
+  return DebugFactory.CreateCompileUnit(LangTag, FileName.c_str(),
+                                        Directory.c_str(),
+                                        version_string, isMain,
+                                        optimize, Flags,
+                                        ObjcRunTimeVer);
+}
+
+/// getOrCreateFile - Get DIFile descriptor.
+DIFile DebugInfo::getOrCreateFile(const char *FullPath) {
+  if (!FullPath) {
+    if (!strcmp (main_input_filename, ""))
+      FullPath = "<stdin>";
+    else
+      FullPath = main_input_filename;
+  }
+
+  // Get source file information.
+  std::string Directory;
+  std::string FileName;
+  DirectoryAndFile(FullPath, Directory, FileName);
+  return DebugFactory.CreateFile(FileName, Directory, TheCU);
 }
 
 /* LLVM LOCAL end (ENTIRE FILE!)  */
