@@ -62,16 +62,11 @@ private:
   const char *PrevFullPath;             // Previous location file encountered.
   int PrevLineNo;                       // Previous location line# encountered.
   BasicBlock *PrevBB;                   // Last basic block encountered.
-  
+  DICompileUnit TheCU;                  // The compile unit.
+
   // Current GCC lexical block (or enclosing FUNCTION_DECL).
   tree_node *CurrentGCCLexicalBlock;	
   
-  // This counter counts debug info for forward referenced subroutine types.
-  // This counter is used to create unique name for such types so that their 
-  // debug info (through MDNodes) is not shared accidently.
-  unsigned FwdTypeCount;
-
-  std::map<std::string, WeakVH > CUCache;
   std::map<tree_node *, WeakVH > TypeCache;
                                         // Cache of previously constructed 
                                         // Types.
@@ -118,9 +113,13 @@ private:
   // by GCC's cfglayout.c:change_scope().
   void change_regions(tree_node *desired, tree_node *grand);
 
-  /// EmitFunctionStart - Constructs the debug code for entering a function -
+  /// CreateSubprogramFromFnDecl - Constructs the debug code for entering a function -
   /// "llvm.dbg.func.start."
-  void EmitFunctionStart(tree_node *FnDecl, Function *Fn, BasicBlock *CurBB);
+  DISubprogram CreateSubprogramFromFnDecl(tree_node *FnDecl);
+
+  /// EmitFunctionStart - Constructs the debug code for entering a function -
+  /// "llvm.dbg.func.start", and pushes it onto the RegionStack.
+  void EmitFunctionStart(tree_node *FnDecl);
 
   /// EmitFunctionEnd - Constructs the debug code for exiting a declarative
   /// region - "llvm.dbg.region.end."
@@ -168,6 +167,9 @@ private:
   DICompileUnit getOrCreateCompileUnit(const char *FullPath,
                                        bool isMain = false);
 
+  /// getOrCreateFile - Get DIFile descriptor.
+  DIFile getOrCreateFile(const char *FullPath);
+
   /// findRegion - Find tree_node N's region.
   DIDescriptor findRegion(tree_node *n);
   
@@ -178,6 +180,13 @@ private:
   /// name is constructred on demand (e.g. C++ destructor) then the name
   /// is stored on the side.
   StringRef getFunctionName(tree_node *FnDecl);
+
+  /// getCU - Get Compile Unit.
+  DICompileUnit getCU() { return TheCU;}
+
+  /// replaceBasicTypesFromPCH - Replace basic type debug info received
+  /// from PCH file.
+  void replaceBasicTypesFromPCH();
 };
 
 } // end namespace llvm
