@@ -37,7 +37,6 @@ Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include "llvm/ADT/StringExtras.h"
 #include "llvm/Support/raw_ostream.h"
 #include <map>
-#undef VISIBILITY_HIDDEN
 
 extern "C" {
 #include "config.h"
@@ -754,7 +753,9 @@ const Type *TypeConverter::ConvertType(tree orig_type) {
     default:
       fprintf(stderr, "Unknown FP type!\n");
       debug_tree(type);
-      abort();        
+      abort();
+    // Half precision FP is a storage only format, use i16 for it.
+    case 16: return SET_TYPE_LLVM(type, Type::getInt16Ty(Context));
     case 32: return SET_TYPE_LLVM(type, Type::getFloatTy(Context));
     case 64: return SET_TYPE_LLVM(type, Type::getDoubleTy(Context));
     case 80: return SET_TYPE_LLVM(type, Type::getX86_FP80Ty(Context));
@@ -789,18 +790,18 @@ const Type *TypeConverter::ConvertType(tree orig_type) {
   case POINTER_TYPE:
   case REFERENCE_TYPE:
   case BLOCK_POINTER_TYPE:
-    if (const PointerType *Ty = cast_or_null<PointerType>(GET_TYPE_LLVM(type))){
+    if (const PointerType *PTy = cast_or_null<PointerType>(GET_TYPE_LLVM(type))){
       // We already converted this type.  If this isn't a case where we have to
       // reparse it, just return it.
       if (PointersToReresolve.empty() || PointersToReresolve.back() != type ||
           ConvertingStruct)
-        return Ty;
+        return PTy;
       
       // Okay, we know that we're !ConvertingStruct and that type is on the end
       // of the vector.  Remove this entry from the PointersToReresolve list and
       // get the pointee type.  Note that this order is important in case the
       // pointee type uses this pointer.
-      assert(Ty->getElementType()->isOpaqueTy() && "Not a deferred ref!");
+      assert(PTy->getElementType()->isOpaqueTy() && "Not a deferred ref!");
       
       // We are actively resolving this pointer.  We want to pop this value from
       // the stack, as we are no longer resolving it.  However, we don't want to
@@ -815,7 +816,7 @@ const Type *TypeConverter::ConvertType(tree orig_type) {
       
       // Note that we know that Ty cannot be resolved or invalidated here.
       const Type *Actual = ConvertType(TREE_TYPE(type));
-      assert(GET_TYPE_LLVM(type) == Ty && "Pointer invalidated!");
+      assert(GET_TYPE_LLVM(type) == PTy && "Pointer invalidated!");
 
       // Restore ConvertingStruct for the caller.
       ConvertingStruct = false;
@@ -824,7 +825,7 @@ const Type *TypeConverter::ConvertType(tree orig_type) {
         Actual = Type::getInt8Ty(Context);  // void* -> sbyte*
       
       // Update the type, potentially updating TYPE_LLVM(type).
-      const OpaqueType *OT = cast<OpaqueType>(Ty->getElementType());
+      const OpaqueType *OT = cast<OpaqueType>(PTy->getElementType());
       const_cast<OpaqueType*>(OT)->refineAbstractTypeTo(Actual);
       return GET_TYPE_LLVM(type);
     } else {
@@ -1098,10 +1099,16 @@ ConvertArgListToFnType(tree type, tree Args, tree static_chain,
     Attrs.push_back(AttributeWithIndex::get(0, RAttributes));
 
   // If this function returns via a shadow argument, the dest loc is passed
-  // in as a pointer.  Mark that pointer as struct-ret and noalias.
+  // in as a pointer.  Mark that pointer as struct-ret.
+  //
+  // It's tempting to want NoAlias here too, however even though llvm-gcc
+  // itself currently always passes a dedicated alloca as the actual argument,
+  // this isn't mandated by the ABI. There are other compilers which don't
+  // always pass a dedicated alloca. Using NoAlias here would make code which
+  // isn't interoperable with that of other compilers.
   if (ABIConverter.isShadowReturn())
     Attrs.push_back(AttributeWithIndex::get(ArgTys.size(),
-                                    Attribute::StructRet | Attribute::NoAlias));
+                                    Attribute::StructRet));
 
   std::vector<const Type*> ScalarArgs;
   if (static_chain) {
@@ -1612,7 +1619,8 @@ struct StructTypeConversionInfo {
     return ~0U;
   }
 
-  void addNewBitField(uint64_t Size, uint64_t FirstUnallocatedByte);
+  void addNewBitField(uint64_t Size, uint64_t Extra,
+                      uint64_t FirstUnallocatedByte);
 
   void dump() const;
 };
@@ -1620,22 +1628,43 @@ struct StructTypeConversionInfo {
 // Add new element which is a bit field. Size is not the size of bit field,
 // but size of bits required to determine type of new Field which will be
 // used to access this bit field.
-void StructTypeConversionInfo::addNewBitField(uint64_t Size,
+// If possible, allocate a field with room for Size+Extra bits.
+void StructTypeConversionInfo::addNewBitField(uint64_t Size, uint64_t Extra,
                                               uint64_t FirstUnallocatedByte) {
 
   // Figure out the LLVM type that we will use for the new field.
   // Note, Size is not necessarily size of the new field. It indicates
   // additional bits required after FirstunallocatedByte to cover new field.
-  const Type *NewFieldTy;
-  if (Size <= 8)
-    NewFieldTy = Type::getInt8Ty(Context);
-  else if (Size <= 16)
-    NewFieldTy = Type::getInt16Ty(Context);
-  else if (Size <= 32)
-    NewFieldTy = Type::getInt32Ty(Context);
-  else {
-    assert(Size <= 64 && "Bitfield too large!");
-    NewFieldTy = Type::getInt64Ty(Context);
+  const Type *NewFieldTy = 0;
+
+  // First try an ABI-aligned field including (some of) the Extra bits.
+  // This field must satisfy Size <= w && w <= XSize.
+  uint64_t XSize = Size + Extra;
+  for (unsigned w = NextPowerOf2(std::min(UINT64_C(64), XSize))/2;
+       w >= Size && w >= 8; w /= 2) {
+    if (TD.isIllegalInteger(w))
+      continue;
+    // Would a w-sized integer field be aligned here?
+    const unsigned a = TD.getABIIntegerTypeAlignment(w);
+    if (FirstUnallocatedByte & (a-1) || a > getGCCStructAlignmentInBytes())
+      continue;
+    // OK, use w-sized integer.
+    NewFieldTy = IntegerType::get(Context, w);
+    break;
+  }
+
+  // Try an integer field that holds Size bits.
+  if (!NewFieldTy) {
+    if (Size <= 8)
+      NewFieldTy = Type::getInt8Ty(Context);
+    else if (Size <= 16)
+      NewFieldTy = Type::getInt16Ty(Context);
+    else if (Size <= 32)
+      NewFieldTy = Type::getInt32Ty(Context);
+    else {
+      assert(Size <= 64 && "Bitfield too large!");
+      NewFieldTy = Type::getInt64Ty(Context);
+    }
   }
 
   // Check that the alignment of NewFieldTy won't cause a gap in the structure!
@@ -1979,7 +2008,34 @@ void TypeConverter::DecodeStructBitField(tree_node *Field,
   // LLVM struct such that there are no holes in the struct where the bitfield
   // is: these holes would make it impossible to statically initialize a global
   // of this type that has an initializer for the bitfield.
-  
+
+  // We want the integer-typed fields as large as possible up to the machine
+  // word size. If there are more bitfields following this one, try to include
+  // them in the same field.
+
+  // Calculate the total number of bits in the continuous group of bitfields
+  // following this one. This is the number of bits that addNewBitField should
+  // try to include.
+  unsigned ExtraSizeInBits = 0;
+  tree LastBitField = 0;
+  for (tree f = TREE_CHAIN(Field); f; f = TREE_CHAIN(f)) {
+    if (TREE_CODE(f) != FIELD_DECL ||
+        TREE_CODE(DECL_FIELD_OFFSET(f)) != INTEGER_CST)
+      break;
+    if (isBitfield(f))
+      LastBitField = f;
+    else {
+      // We can use all this bits up to the next non-bitfield.
+      LastBitField = 0;
+      ExtraSizeInBits = getFieldOffsetInBits(f) - EndBitOffset;
+      break;
+    }
+  }
+  // Record ended in a bitfield? Use all of the last byte.
+  if (LastBitField)
+    ExtraSizeInBits = RoundUpToAlignment(getFieldOffsetInBits(LastBitField) +
+      TREE_INT_CST_LOW(DECL_SIZE(LastBitField)), 8) - EndBitOffset;
+
   // Compute the number of bits that we need to add to this struct to cover
   // this field.
   uint64_t FirstUnallocatedByte = Info.getEndUnallocatedByte();
@@ -1993,7 +2049,7 @@ void TypeConverter::DecodeStructBitField(tree_node *Field,
       // This field starts at byte boundry. Need to allocate space
       // for additional bytes not yet allocated.
       unsigned NumBitsToAdd = FieldSizeInBits - AvailableBits;
-      Info.addNewBitField(NumBitsToAdd, FirstUnallocatedByte);
+      Info.addNewBitField(NumBitsToAdd, ExtraSizeInBits, FirstUnallocatedByte);
       return;
     }
 
@@ -2017,7 +2073,7 @@ void TypeConverter::DecodeStructBitField(tree_node *Field,
       for (unsigned idx = 0; idx < (prevFieldTypeSizeInBits/8); ++idx)
 	FirstUnallocatedByte--;
     }
-    Info.addNewBitField(NumBitsRequired, FirstUnallocatedByte);
+    Info.addNewBitField(NumBitsRequired, ExtraSizeInBits, FirstUnallocatedByte);
     // Do this after adding Field.
     Info.lastFieldStartsAtNonByteBoundry(true);
     return;
@@ -2051,7 +2107,7 @@ void TypeConverter::DecodeStructBitField(tree_node *Field,
   }
 
   // Now, Field starts at FirstUnallocatedByte and everything is aligned.
-  Info.addNewBitField(FieldSizeInBits, FirstUnallocatedByte);
+  Info.addNewBitField(FieldSizeInBits, ExtraSizeInBits, FirstUnallocatedByte);
 }
 
 /// UnionHasOnlyZeroOffsets - Check if a union type has only members with
@@ -2123,7 +2179,7 @@ void TypeConverter::SelectUnionMember(tree type,
 
     if (isBitfield(UnionField)) {
       unsigned FieldSizeInBits = TREE_INT_CST_LOW(DECL_SIZE(UnionField));
-      Info.addNewBitField(FieldSizeInBits, 0);
+      Info.addNewBitField(FieldSizeInBits, 0, 0);
     } else {
       Info.allFieldsAreNotBitFields();
       Info.addElement(UnionTy, 0, Info.getTypeSize(UnionTy));
