@@ -441,9 +441,7 @@ layout_decl (tree decl, unsigned int known_align)
 	    = MIN (DECL_ALIGN (decl), (unsigned) BIGGEST_FIELD_ALIGNMENT);
 #endif
 #ifdef ADJUST_FIELD_ALIGN
-/* APPLE LOCAL begin mainline 2006-10-31 PR 23067, radar 4869885 */
 	  DECL_ALIGN (decl) = ADJUST_FIELD_ALIGN (decl, DECL_ALIGN (decl));
-/* APPLE LOCAL end mainline 2006-10-31 PR 23067, radar 4869885 */
 #endif
 	}
 
@@ -535,6 +533,10 @@ record_layout_info
 start_record_layout (tree t)
 {
   record_layout_info rli = xmalloc (sizeof (struct record_layout_info_s));
+  /* APPLE LOCAL begin 5946347 ms_struct support */
+  unsigned biggest_alignment = targetm.ms_bitfield_layout_p (t) ?
+    BIGGEST_MS_STRUCT_ALIGNMENT
+    : BIGGEST_ALIGNMENT;
 
   rli->t = t;
 
@@ -543,7 +545,8 @@ start_record_layout (tree t)
      one-byte alignment.  */
   rli->record_align = MAX (BITS_PER_UNIT, TYPE_ALIGN (t));
   rli->unpacked_align = rli->record_align;
-  rli->offset_align = MAX (rli->record_align, BIGGEST_ALIGNMENT);
+  rli->offset_align = MAX (rli->record_align, biggest_alignment);
+  /* APPLE LOCAL end 5946347 ms_struct support */
 
 #ifdef STRUCTURE_SIZE_BOUNDARY
 /* APPLE LOCAL begin ARM Macintosh alignment */
@@ -709,9 +712,7 @@ update_alignment_for_field (record_layout_info rli, tree field,
 #ifdef ADJUST_FIELD_ALIGN
   if (! user_align && TREE_CODE (rli->t) == RECORD_TYPE)
     desired_align = 
-/* APPLE LOCAL begin mainline 2006-10-31 PR 23067, radar 4869885 */
       ADJUST_FIELD_ALIGN (field, desired_align);
-/* APPLE LOCAL end mainline 2006-10-31 PR 23067, radar 4869885 */
 #endif
   /* APPLE LOCAL end Macintosh alignment 2002-5-24 --ff */
 
@@ -733,7 +734,8 @@ update_alignment_for_field (record_layout_info rli, tree field,
 		 && DECL_BIT_FIELD_TYPE (rli->prev_field)
 		 && ! integer_zerop (DECL_SIZE (rli->prev_field)))))
 	{
-	  unsigned int type_align = TYPE_ALIGN (type);
+	  /* APPLE LOCAL 5946347 ms_struct support */
+	  unsigned int type_align = TARGET_FIELD_MS_STRUCT_ALIGN (field);
 	  type_align = MAX (type_align, desired_align);
 	  if (maximum_field_alignment != 0)
 	    type_align = MIN (type_align, maximum_field_alignment);
@@ -754,9 +756,7 @@ update_alignment_for_field (record_layout_info rli, tree field,
 
 #ifdef ADJUST_FIELD_ALIGN
 	  if (! TYPE_USER_ALIGN (type))
-/* APPLE LOCAL begin mainline 2006-10-31 PR 23067, radar 4869885 */
 	    type_align = ADJUST_FIELD_ALIGN (field, type_align);
-/* APPLE LOCAL end mainline 2006-10-31 PR 23067, radar 4869885 */
 #endif
 
 	  /* Targets might chose to handle unnamed and hence possibly
@@ -1008,9 +1008,7 @@ place_field (record_layout_info rli, tree field)
 
 #ifdef ADJUST_FIELD_ALIGN
       if (! TYPE_USER_ALIGN (type))
-/* APPLE LOCAL begin mainline 2006-10-31 PR 23067, radar 4869885 */
 	type_align = ADJUST_FIELD_ALIGN (field, type_align);
-/* APPLE LOCAL end mainline 2006-10-31 PR 23067, radar 4869885 */
 #endif
 
       /* A bit field may not span more units of alignment of its type
@@ -1042,9 +1040,7 @@ place_field (record_layout_info rli, tree field)
 
 #ifdef ADJUST_FIELD_ALIGN
       if (! TYPE_USER_ALIGN (type))
-/* APPLE LOCAL begin mainline 2006-10-31 PR 23067, radar 4869885 */
 	type_align = ADJUST_FIELD_ALIGN (field, type_align);
-/* APPLE LOCAL end mainline 2006-10-31 PR 23067, radar 4869885 */
 #endif
 
       if (maximum_field_alignment != 0)
@@ -1139,17 +1135,6 @@ place_field (record_layout_info rli, tree field)
 				       TYPE_SIZE (type),
 				       bitsize_int (rli->remaining_in_alignment)));
 		      rli->prev_field = field;
-		      /* MERGE FIXME 5416334 audit for correctness wrt:
-
-			 2006-07-15  Kaz Kojima  <kkojima@gcc.gnu.org>
-
-			 PR middle-end/28160
-			 * stor-layout.c (place_field): Take the bit field with
-			 an excessive size into account in the ms-bitfiled case.
-
-			 PR middle-end/28161
-			 * stor-layout.c (place_field): Use DECL_BIT_FIELD_TYPE of
-			 the previous bit field.  */
 		      if (typesize < bitsize)
 			rli->remaining_in_alignment = 0;
 		      else
@@ -1288,7 +1273,8 @@ place_field (record_layout_info rli, tree field)
 	    }
 
 	  /* Now align (conventionally) for the new type.  */
-	  type_align = TYPE_ALIGN (TREE_TYPE (field));
+	  /* APPLE LOCAL 5946347 ms_struct support */
+	  type_align = TARGET_FIELD_MS_STRUCT_ALIGN (field);
 
 	  if (maximum_field_alignment != 0)
 	    type_align = MIN (type_align, maximum_field_alignment);
@@ -1900,6 +1886,8 @@ layout_type (tree type)
 
     case POINTER_TYPE:
     case REFERENCE_TYPE:
+    /* APPLE LOCAL blocks */
+    case BLOCK_POINTER_TYPE:
       {
 
 	enum machine_mode mode = ((TREE_CODE (type) == REFERENCE_TYPE
