@@ -84,12 +84,6 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #define TARGET_ASM_TTYPE hook_bool_rtx_false
 #endif
 
-/* LLVM LOCAL begin */
-#ifndef TARGET_ARM_TTYPE
-#define TARGET_ASM_TTYPE hook_bool_rtx_false
-#endif
-/* LLVM LOCAL end */
-
 #ifndef TARGET_ASM_ASSEMBLE_VISIBILITY
 #define TARGET_ASM_ASSEMBLE_VISIBILITY default_assemble_visibility
 #endif
@@ -343,9 +337,16 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
    TARGET_SCHED_SET_SCHED_FLAGS}
 
 #define TARGET_VECTORIZE_BUILTIN_MASK_FOR_LOAD 0
+/* APPLE LOCAL begin mainline 4.2 5569774 */
+#define TARGET_VECTOR_ALIGNMENT_REACHABLE \
+  default_builtin_vector_alignment_reachable
+/* APPLE LOCAL end mainline 4.2 5569774 */
 
 #define TARGET_VECTORIZE                                                \
-  {TARGET_VECTORIZE_BUILTIN_MASK_FOR_LOAD}
+  /* APPLE LOCAL begin mainline 4.2 5569774 */				\
+  {TARGET_VECTORIZE_BUILTIN_MASK_FOR_LOAD,				\
+   TARGET_VECTOR_ALIGNMENT_REACHABLE}
+  /* APPLE LOCAL end mainline 4.2 5569774 */
 
 #define TARGET_DEFAULT_TARGET_FLAGS 0
 
@@ -448,7 +449,8 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #define TARGET_ALIGN_ANON_BITFIELD hook_bool_void_false
 #define TARGET_NARROW_VOLATILE_BITFIELD hook_bool_void_false
 #define TARGET_RTX_COSTS hook_bool_rtx_int_int_intp_false
-#define TARGET_MANGLE_FUNDAMENTAL_TYPE hook_constcharptr_tree_null
+/* APPLE LOCAL mangle_type 7105099 */	\
+#define TARGET_MANGLE_TYPE hook_constcharptr_tree_null
 #define TARGET_ALLOCATE_INITIAL_VALUE NULL
 
 #ifndef TARGET_INIT_LIBFUNCS
@@ -474,6 +476,9 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #define TARGET_INVALID_CONVERSION hook_constcharptr_tree_tree_null
 #define TARGET_INVALID_UNARY_OP hook_constcharptr_int_tree_null
 #define TARGET_INVALID_BINARY_OP hook_constcharptr_int_tree_tree_null
+/* LLVM LOCAL begin */
+#define TARGET_TYPE_PROMOTES_TO hook_tree_tree_null
+/* LLVM LOCAL end */
 
 #define TARGET_FIXED_CONDITION_CODE_REGS hook_bool_uintp_uintp_false
 
@@ -504,16 +509,6 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 #define TARGET_ARM_EABI_UNWINDER false
 
-/* APPLE LOCAL begin mainline 2005-10-12 */
-#ifndef TARGET_VALID_DLLIMPORT_ATTRIBUTE_P
-#define TARGET_VALID_DLLIMPORT_ATTRIBUTE_P hook_bool_tree_true
-#endif
-/* APPLE LOCAL end mainline 2005-10-12 */
-
-/* LLVM LOCAL begin */
-#define TARGET_ARM_EABI_UNWINDER false
-/* LLVM LOCAL end */
-
 #define TARGET_PROMOTE_FUNCTION_ARGS hook_bool_tree_false
 #define TARGET_PROMOTE_FUNCTION_RETURN hook_bool_tree_false
 #define TARGET_PROMOTE_PROTOTYPES hook_bool_tree_false
@@ -593,11 +588,6 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 #define TARGET_SECONDARY_RELOAD default_secondary_reload
 #endif
 
-/* LLVM LOCAL begin */
-#ifndef TARGET_UNWIND_TABLES_DEFAULT
-#define TARGET_UNWIND_TABLES_DEFAULT false
-#endif
-/* LLVM LOCAL end */
 
 /* C++ specific.  */
 #ifndef TARGET_CXX_GUARD_TYPE
@@ -698,7 +688,9 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
   /* APPLE LOCAL end constant cfstrings */	\
   TARGET_RESOLVE_OVERLOADED_BUILTIN,		\
   TARGET_FOLD_BUILTIN,				\
-  TARGET_MANGLE_FUNDAMENTAL_TYPE,		\
+  /* APPLE LOCAL begin mangle_type 7105099 */	\
+  TARGET_MANGLE_TYPE,				\
+  /* APPLE LOCAL end mangle_type 7105099 */	\
   TARGET_INIT_LIBFUNCS,				\
   TARGET_SECTION_TYPE_FLAGS,			\
   TARGET_CANNOT_MODIFY_JUMPS_P,			\
@@ -750,6 +742,9 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
   TARGET_INVALID_CONVERSION,			\
   TARGET_INVALID_UNARY_OP,			\
   TARGET_INVALID_BINARY_OP,			\
+  /* LLVM LOCAL begin */			\
+  TARGET_TYPE_PROMOTES_TO,		        \
+  /* LLVM LOCAL end   */			\
   TARGET_SECONDARY_RELOAD,			\
   TARGET_CXX,					\
   TARGET_EXTRA_LIVE_ON_ENTRY,                    \
@@ -773,5 +768,12 @@ Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
   /* APPLE LOCAL end radar 5155743, mainline candidate */	\
 }
 
+/* LLVM LOCAL begin sysroot */
+#ifdef ENABLE_LLVM
+#ifndef TARGET_BUILD_SYSROOT_PATH
+#define TARGET_BUILD_SYSROOT_PATH(S,P) default_build_sysroot_path((S), (P))
+#endif
+#endif
+/* LLVM LOCAL end sysroot */
 #include "hooks.h"
 #include "targhooks.h"
