@@ -375,6 +375,12 @@ struct gcc_target
        by the vectorizer, and return the decl of the target builtin
        function.  */
     tree (* builtin_mask_for_load) (void);
+
+    /* APPLE LOCAL begin mainline 4.2 5569774 */
+    /* Return true if vector alignment is reachable (by peeling N
+      interations) for the given type.  */
+     bool (* vector_alignment_reachable) (tree, bool);
+    /* APPLE LOCAL end mainline 4.2 5569774 */
   } vectorize;
 
   /* The initial value of target_flags.  */
@@ -463,10 +469,12 @@ struct gcc_target
   /* Fold a target-specific builtin.  */
   tree (* fold_builtin) (tree fndecl, tree arglist, bool ignore);
 
-  /* For a vendor-specific fundamental TYPE, return a pointer to
-     a statically-allocated string containing the C++ mangling for
-     TYPE.  In all other cases, return NULL.  */
-  const char * (* mangle_fundamental_type) (tree type);
+  /* APPLE LOCAL begin mangle_type 7105099 */
+  /* For a vendor-specific TYPE, return a pointer to a statically-allocated
+     string containing the C++ mangling for TYPE.  In all other cases, return
+     NULL.  */
+  const char * (* mangle_type) (tree type);
+  /* APPLE LOCAL end mangle_type 7105099 */
 
   /* Make any adjustments to libfunc names needed for this target.  */
   void (* init_libfuncs) (void);
@@ -762,6 +770,12 @@ struct gcc_target
      is not permitted on TYPE1 and TYPE2, NULL otherwise.  */
   const char *(*invalid_binary_op) (int op, tree type1, tree type2);
 
+  /* LLVM LOCAL begin */
+  /* Returns whether the type should be promoted to some other type (e.g. HF ->
+     SF conversion on ARM) */
+  tree (*type_promotes_to) (tree ty);
+  /* LLVM LOCAL end */
+
   /* Return the class for a secondary reload, and fill in extra information.  */
   enum reg_class (*secondary_reload) (bool, rtx, enum reg_class,
 				      enum machine_mode,
