@@ -6,17 +6,13 @@
 typedef struct dispatch_queue_s *dispatch_queue_t;
 typedef struct dispatch_item_s *dispatch_item_t;
 
-typedef void (^dispatch_block_t)(dispatch_item_t item);
+dispatch_item_t dispatch_call(dispatch_queue_t queue,
+                              dispatch_legacy_block_t work,
+                              dispatch_legacy_block_t completion);
 
-void dispatch_call(dispatch_queue_t queue,
- dispatch_block_t work,
- dispatch_block_t completion,
- void* context,
- dispatch_item_t *item);
-
-void dispatch_apply_wait(dispatch_block_t work,
-     unsigned iterations,
-     void *context);
+void dispatch_apply_wait(dispatch_legacy_block_t work,
+                         unsigned iterations,
+                         void *context);
 
 
 
@@ -34,15 +30,14 @@ void dispatch_apply_wait(dispatch_block_t work,
 {
   dispatch_queue_t queue;
 
-  dispatch_call(queue,
+  dispatch_call(queue, /* { dg-warning "deprecated" } */
     ^(dispatch_item_t item) {
       dispatch_apply_wait(^(dispatch_item_t item) {
                             [NSValue valueWithPointer:self];
                           },
                           10,NULL);
     },
-    ^(dispatch_item_t item) {NSLog(@"test");},
-    NULL, NULL);
+    ^(dispatch_item_t item) {NSLog(@"test");});
 }
 
 
