@@ -4,7 +4,7 @@
 /* { dg-skip-if "" { powerpc*-*-darwin* } { "-m64" } { "" } } */
 
 #include <stdio.h>
-void * _NSConcreteStackBlock;
+void * _NSConcreteStackBlock[32];
 void _Block_byref_assign_copy(void * dst, void *src){}
 void _Block_byref_release(void*src){}
 
