@@ -4,7 +4,7 @@
 /* { dg-skip-if "" { powerpc*-*-darwin* } { "-m64" } { "" } } */
 
 #include <stdio.h>
-void * _NSConcreteStackBlock;
+void * _NSConcreteStackBlock[32];
 #ifdef __cplusplus
 extern "C" void exit(int);
 #else
