@@ -4,7 +4,7 @@
 /* { dg-skip-if "" { powerpc*-*-darwin* } { "-m64" } { "" } } */
 
 #import <Foundation/Foundation.h>
-void * _NSConcreteStackBlock;
+void * _NSConcreteStackBlock[32];
 
 int GlobalInt = 0;
 int GlobalInt2 = 0;
