@@ -84,7 +84,9 @@ Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
 #include "tree-mudflap.h"
 
 /* LLVM LOCAL begin */
+#ifdef ENABLE_LLVM
 #include "llvm.h"
+#endif
 /* LLVM LOCAL end */
 
 #if defined (DWARF2_UNWIND_INFO) || defined (DWARF2_DEBUGGING_INFO)
@@ -257,10 +259,10 @@ int flag_signed_char;
 int flag_short_enums;
 
 /* LLVM LOCAL begin */
-
+#ifdef ENABLE_LLVM
 /* Options to pass directly into the LLVM backend. */
 const char *llvm_optns = 0;
-
+#endif
 /* LLVM LOCAL end */
 
 /* APPLE LOCAL begin -fast or -fastf or -fastcp */
@@ -397,9 +399,9 @@ int align_labels_log;
 int align_labels_max_skip;
 int align_functions_log;
 
-/* Like align_functions_log above, but used by front-ends to force the
-   minimum function alignment.  Zero means no alignment is forced.  */
-int force_align_functions_log;
+/* APPLE LOCAL begin mainline aligned functions 5933878 */
+/* Removed force_align_functions_log.  */
+/* APPLE LOCAL end mainline aligned functions 5933878 */
 
 typedef struct
 {
@@ -938,6 +940,11 @@ warn_deprecated_use (tree node)
   if (node == 0 || !warn_deprecated_decl)
     return;
 
+  /* APPLE LOCAL begin radar 4746503 */
+  if (current_function_decl && TREE_DEPRECATED (current_function_decl))
+    return;
+  /* APPLE LOCAL end radar 4746503 */
+
   if (DECL_P (node))
     {
       expanded_location xloc = expand_location (DECL_SOURCE_LOCATION (node));
@@ -1184,6 +1191,8 @@ compile_file (void)
 #else
   if (!flag_pch_file)
     llvm_asm_file_end();
+  /* Release LLVM global state. */
+  llvm_call_llvm_shutdown();
 #endif
   /* LLVM LOCAL end */
 }
@@ -1224,6 +1233,14 @@ decode_d_option (const char *arg)
 	setup_core_dumping();
 	break;
 
+      /* LLVM LOCAL begin */
+#ifdef ENABLE_LLVM
+      case 'M':
+         /* Ignore -dM. This is overloaded for rtl pass that 
+            does not exist in llvm-gcc.  */
+         break;
+#endif
+      /* LLVM LOCAL end */
       case 'a':
       default:
 	if (!enable_rtl_dump_file (c))
@@ -1705,8 +1722,19 @@ general_init (const char *argv0)
   /* Register the language-independent parameters.  */
   add_params (lang_independent_params, LAST_PARAM);
 
-  /* This must be done after add_params but before argument processing.  */
-  init_ggc_heuristics();
+  /* APPLE LOCAL begin retune gc params 6124839 */
+  { int i = 0;
+    bool opt = false;
+    while (save_argv[++i])
+      {
+	if (strncmp (save_argv[i], "-O", 2) == 0
+	    && strcmp (save_argv[i], "-O0") != 0)
+	  opt = true;
+      }
+    /* This must be done after add_params but before argument processing.  */
+    init_ggc_heuristics(opt);
+  }
+  /* APPLE LOCAL end retune gc params 6124839 */
   init_optimization_passes ();
 }
 
@@ -1817,6 +1845,14 @@ process_options (void)
   if (flag_value_profile_transformations)
     flag_profile_values = 1;
 
+  /* LLVM LOCAL begin */
+#ifdef ENABLE_LLVM
+  /* Disable verbose_asm flag if -emit-llvm is used, it's totally bogus then */
+  if (emit_llvm || emit_llvm_bc)
+    flag_verbose_asm = 0;
+#endif
+  /* LLVM LOCAL end */
+
   /* Warn about options that are not supported on this machine.  */
 #ifndef INSN_SCHEDULING
   if (flag_schedule_insns || flag_schedule_insns_after_reload)
@@ -1890,6 +1926,8 @@ process_options (void)
 
   /* LLVM LOCAL begin */
 #ifdef ENABLE_LLVM
+  /* write_symbols set up debug_hooks. llvm-gcc does not use this hooks
+     to emit debug info. */
   write_symbols = NO_DEBUG;
 #endif
   /* LLVM LOCAL end */
@@ -2011,6 +2049,12 @@ process_options (void)
   if (flag_cx_limited_range)
     flag_complex_method = 0;
 
+  /* APPLE LOCAL begin stack-protector default 5095227 */
+  /* Unless the target chooses otherwise, default stack protection to off.  */
+  if (flag_stack_protect == -1)
+    flag_stack_protect = 0;
+  /* APPLE LOCAL end stack-protector default 5095227 */
+
   /* APPLE LOCAL begin optimization pragmas 3124235/3420242 */
   cl_pf_opts_cooked = cl_pf_opts;
   /* APPLE LOCAL end optimization pragmas 3124235/3420242 */
@@ -2052,6 +2096,9 @@ backend_init (void)
 
   /* LLVM LOCAL begin */
 #ifdef ENABLE_LLVM
+  /* note: stor-layout.c:layout_type() uses have_regs_of_mode[] to initialize
+     TYPE_MODE field for user specified vector types. */
+  init_regs ();
   /* note: we need init_emit_once above to initialize ptr_mode and friends. */
   llvm_initialize_backend();
   return;
@@ -2104,8 +2151,7 @@ lang_dependent_init (const char *name)
   llvm_lang_dependent_init(name);
   init_eh();
   return 1; /* don't initialize the RTL backend */
-#endif
-  /* LLVM LOCAL end */
+#else
   
   /* These create various _DECL nodes, so need to be called after the
      front end is initialized.  */
@@ -2141,6 +2187,8 @@ lang_dependent_init (const char *name)
   timevar_pop (TV_SYMOUT);
 
   return 1;
+#endif
+  /* LLVM LOCAL end */
 }
 
 /* Clean up: close opened files, etc.  */
