@@ -1024,6 +1024,12 @@ setup_one_parameter (copy_body_data *id, tree p, tree value, tree fn,
      to just use the argument value.  */
   if (TREE_READONLY (p)
       && !TREE_ADDRESSABLE (p)
+/* LLVM LOCAL begin fix handling of parameters with variably modified types */
+#ifdef ENABLE_LLVM
+      /* We need the copy if it has a variably modified type. */
+      && !variably_modified_type_p (TREE_TYPE (p), id->src_fn)
+#endif
+/* LLVM LOCAL end */
       && value && !TREE_SIDE_EFFECTS (value))
     {
       /* We may produce non-gimple trees by adding NOPs or introduce
@@ -1049,6 +1055,16 @@ setup_one_parameter (copy_body_data *id, tree p, tree value, tree fn,
      function.  */
   var = copy_decl_to_var (p, id);
 
+/* LLVM LOCAL begin fix handling of parameters with variably modified types */
+#ifdef ENABLE_LLVM
+  /* But, we must remap variably modified types, which depend on local
+     variables in this function and are NOT visible to the calling
+     function. */
+  if (variably_modified_type_p (TREE_TYPE (p), id->src_fn))
+    TREE_TYPE (var) = remap_type(TREE_TYPE (p), id);    
+#endif
+/* LLVM LOCAL end */
+
   /* See if the frontend wants to pass this by invisible reference.  If
      so, our new VAR_DECL will have REFERENCE_TYPE, and we need to
      replace uses of the PARM_DECL with dereferences.  */
@@ -1261,16 +1277,16 @@ declare_return_variable (copy_body_data *id, tree return_slot_addr,
 	{
 	  var = modify_dest;
 	  use = NULL;
-    
-    /* LLVM LOCAL begin */
+
+          /* LLVM LOCAL begin */
 #ifdef ENABLE_LLVM
-    /* The return node can suddenly become multiple assignment.  Because of this
-     * conservatively don't consider this thing for SSA form anymore.
-    */
-    if (DECL_P (var))
-      DECL_GIMPLE_FORMAL_TEMP_P (var) = 0;
+          /* The return node may not dominate all of its uses.  Because of this,
+             conservatively don't consider it for SSA form anymore.  */
+          if (TREE_CODE (var) == VAR_DECL)
+            DECL_GIMPLE_FORMAL_TEMP_P (var) = 0;
 #endif
     /* LLVM LOCAL end */
+
 	  goto done;
 	}
     }
@@ -1509,6 +1525,11 @@ inlinable_function_p (tree fn)
 {
   bool inlinable = true;
 
+/* LLVM LOCAL begin turn off gcc inliner */
+#ifdef ENABLE_LLVM
+  DECL_UNINLINABLE (fn) = 1;
+#endif
+/* LLVM LOCAL end turn off gcc inliner */
   /* If we've already decided this function shouldn't be inlined,
      there's no need to check again.  */
   if (DECL_UNINLINABLE (fn))
@@ -1570,7 +1591,7 @@ inlinable_function_p (tree fn)
 
   /* Squirrel away the result so that we don't have to check again.  */
   DECL_UNINLINABLE (fn) = !inlinable;
-  
+
   return inlinable;
 }
 
@@ -2021,12 +2042,16 @@ expand_call_inline (basic_block bb, tree stmt, tree *tp, void *data)
      inlining.  */
   if (!cgraph_inline_p (cg_edge, &reason))
     {
+/* LLVM LOCAL begin */
+#ifndef ENABLE_LLVM
       if (lookup_attribute ("always_inline", DECL_ATTRIBUTES (fn))
 	  /* Avoid warnings during early inline pass. */
 	  && (!flag_unit_at_a_time || cgraph_global_info_ready))
 	{
-	  sorry ("inlining failed in call to %q+F: %s", fn, reason);
-	  sorry ("called from here");
+	  /* APPLE LOCAL begin wording 4598393 */
+	  error ("%<always_inline%> function could not be inlined in call to %q+F: %s", fn, reason);
+	  error ("called from here");
+	  /* APPLE LOCAL end wording 4598393 */
 	}
       else if (warn_inline && DECL_DECLARED_INLINE_P (fn)
 	       && !DECL_IN_SYSTEM_HEADER (fn)
@@ -2039,6 +2064,8 @@ expand_call_inline (basic_block bb, tree stmt, tree *tp, void *data)
 		   fn, reason);
 	  warning (OPT_Winline, "called from here");
 	}
+#endif
+/* LLVM LOCAL end */
       goto egress;
     }
   fn = cg_edge->callee->decl;
@@ -2637,7 +2664,7 @@ copy_decl_for_dup_finish (copy_body_data *id, tree decl, tree copy)
 #ifdef ENABLE_LLVM
     SET_DECL_LLVM (copy, 0);
 #else
-    SET_DECL_RTL (copy, NULL_RTX); 
+    SET_DECL_RTL (copy, NULL_RTX);
 #endif
   /* LLVM_LOCAL end */
   
@@ -2830,7 +2857,7 @@ tree_function_versioning (tree old_decl, tree new_decl, varray_type tree_map,
   if (!update_clones)
     DECL_NAME (new_decl) = create_tmp_var_name (NULL);
   /* LLVM LOCAL begin */
-  #ifndef ENABLE_LLVM
+#ifndef ENABLE_LLVM
   /* Create a new SYMBOL_REF rtx for the new name. */
   if (DECL_RTL (old_decl) != NULL)
     {
@@ -2839,7 +2866,7 @@ tree_function_versioning (tree old_decl, tree new_decl, varray_type tree_map,
 	gen_rtx_SYMBOL_REF (GET_MODE (XEXP (DECL_RTL (old_decl), 0)),
 			    IDENTIFIER_POINTER (DECL_NAME (new_decl)));
     }
-  #endif
+#endif
   /* LLVM LOCAL end */
     
   /* Prepare the data structures for the tree copy.  */
